@@ -1,6 +1,5 @@
-#! /usr/bin/perl
-# -*- Perl -*-
-# Copyright (C) 1989-2018 Free Software Foundation, Inc.
+#!@PERL@
+# Copyright (C) 1989-2020 Free Software Foundation, Inc.
 #
 # This file is part of groff.
 #
@@ -18,6 +17,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 use strict;
+use warnings;
+
+(my $progname = $0) =~s @.*/@@;
+
 # runs groff in safe mode, that seems to be the default
 # installation now. That means that I have to fix all nice
 # features outside groff. Sigh.
@@ -25,6 +28,17 @@ use strict;
 # of security holes.
 
 my $no_exec;
+
+if (grep(/^--help$/, @ARGV)) {
+	print "usage: mmroff [-x] [groff-option ...] [file ...]\n";
+	exit;
+}
+
+if (grep(/^--version$/, @ARGV)) {
+	print "mmroff (groff) @VERSION@\n";
+	exit;
+}
+
 # check for -x and remove it
 if (grep(/^-x$/, @ARGV)) {
 	$no_exec++;
@@ -84,12 +98,17 @@ while(<MACRO>) {
 }
 close(MACRO);
 
+sub Die {
+	print STDERR "$progname: fatal error: @_\n";
+	exit 1;
+}
 
 if ($rfilename) {
 	push(@out, ".nr pict*max-height $max_height\n") if defined $max_height;
 	push(@out, ".nr pict*max-width $max_width\n") if defined $max_width;
 
-	open(OUT, ">$rfilename") || "create $rfilename:$!";
+	open(OUT, ">$rfilename")
+		or &Die("unable to create $rfilename:$!");
 	print OUT '.\" references', "\n";
 	my $i;
 	for $i (@out) {
@@ -104,7 +123,7 @@ exit system($run_macro);
 sub print_index {
 	my ($f, $ind, $macro) = @_;
 
-	open(OUT, ">$f") || "create $f:$!";
+	open(OUT, ">$f") or &Die("unable to create $f:$!");
 	my $i;
 	for $i (sort @$ind) {
 		if ($macro) {
@@ -155,8 +174,6 @@ sub psbb {
 
 
 1;
-########################################################################
-### Emacs settings
 # Local Variables:
 # mode: CPerl
 # End:
