@@ -1,5 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2014  Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -15,7 +14,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+
+#include <assert.h>
 
 #include "driver.h"
 #include "nonposix.h"
@@ -85,7 +86,7 @@ void dvi_font::handle_unknown_font_command(const char *command,
   if (strcmp(command, "checksum") == 0) {
     if (arg == 0)
       fatal_with_file_and_line(filename, lineno,
-			       "`checksum' command requires an argument");
+			       "'checksum' command requires an argument");
     checksum = int(strtol(arg, &ptr, 10));
     if (checksum == 0 && ptr == arg) {
       fatal_with_file_and_line(filename, lineno, "bad checksum");
@@ -94,7 +95,7 @@ void dvi_font::handle_unknown_font_command(const char *command,
   else if (strcmp(command, "designsize") == 0) {
     if (arg == 0)
       fatal_with_file_and_line(filename, lineno,
-			       "`designsize' command requires an argument");
+			       "'designsize' command requires an argument");
     design_size = int(strtol(arg, &ptr, 10));
     if (design_size == 0 && ptr == arg) {
       fatal_with_file_and_line(filename, lineno, "bad design size");
@@ -316,23 +317,23 @@ void dvi_printer::set_color(color *col)
     break;
   case RGB:
     sprintf(buf, "color rgb %.3g %.3g %.3g",
-		 double(Red) / color::MAX_COLOR_VAL,
-		 double(Green) / color::MAX_COLOR_VAL,
-		 double(Blue) / color::MAX_COLOR_VAL);
+		 double(Red) / double(color::MAX_COLOR_VAL),
+		 double(Green) / double(color::MAX_COLOR_VAL),
+		 double(Blue) / double(color::MAX_COLOR_VAL));
     break;
   case CMY:
     col->get_cmyk(&Cyan, &Magenta, &Yellow, &Black);
     // fall through
   case CMYK:
     sprintf(buf, "color cmyk %.3g %.3g %.3g %.3g",
-		 double(Cyan) / color::MAX_COLOR_VAL,
-		 double(Magenta) / color::MAX_COLOR_VAL,
-		 double(Yellow) / color::MAX_COLOR_VAL,
-		 double(Black) / color::MAX_COLOR_VAL);
+		 double(Cyan) / double(color::MAX_COLOR_VAL),
+		 double(Magenta) / double(color::MAX_COLOR_VAL),
+		 double(Yellow) / double(color::MAX_COLOR_VAL),
+		 double(Black) / double(color::MAX_COLOR_VAL));
     break;
   case GRAY:
     sprintf(buf, "color gray %.3g",
-		 double(Gray) / color::MAX_COLOR_VAL);
+		 double(Gray) / double(color::MAX_COLOR_VAL));
     break;
   }
   do_special(buf);
@@ -694,7 +695,7 @@ void draw_dvi_printer::fill_next(const environment *env)
     env->fill->get_gray(&g);
   }
   char buf[256];
-  sprintf(buf, "sh %.3g", 1 - double(g)/color::MAX_COLOR_VAL);
+  sprintf(buf, "sh %.3g", 1 - double(g) / double(color::MAX_COLOR_VAL));
   do_special(buf);
 }
 
@@ -885,7 +886,7 @@ void draw_dvi_printer::draw(int code, int *p, int np, const environment *env)
       break;
     }
   default:
-    error("unrecognised drawing command `%1'", char(code));
+    error("unrecognised drawing command '%1'", char(code));
     break;
   }
 }
@@ -935,7 +936,7 @@ int main(int argc, char **argv)
     case 'p':
       if (!font::scan_papersize(optarg, 0,
 				&user_paper_length, &user_paper_width))
-	error("invalid custom paper size `%1' ignored", optarg);
+	error("ignoring invalid paper format '%1'", optarg);
       break;
     case 'v':
       {
@@ -946,7 +947,7 @@ int main(int argc, char **argv)
     case 'w':
       if (sscanf(optarg, "%d", &linewidth) != 1
 	  || linewidth < 0 || linewidth > 1000) {
-	error("bad line width");
+	error("invalid line width '%1' ignored", optarg);
 	linewidth = DEFAULT_LINEWIDTH;
       }
       break;
@@ -973,6 +974,15 @@ int main(int argc, char **argv)
 
 static void usage(FILE *stream)
 {
-  fprintf(stream, "usage: %s [-dv] [-F dir] [-w n] [files ...]\n",
-	  program_name);
+  fprintf(stream,
+"usage: %s [-dl] [-F dir] [-p paper-format] [-w n] [file ...]\n"
+"usage: %s {-v | --version}\n"
+"usage: %s --help\n",
+          program_name, program_name, program_name);
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
