@@ -1,6 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 2000-2014
- * Free Software Foundation, Inc.
+/* Copyright (C) 2000-2020 Free Software Foundation, Inc.
  *
  *  Gaius Mulley (gaius@glam.ac.uk) wrote html-text.cpp
  *
@@ -24,7 +23,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "driver.h"
 #include "stringclass.h"
@@ -40,9 +39,6 @@ along with this program. If not, see <http://www.gnu.org/licenses/>. */
 
 #include "html-text.h"
 
-#undef DEBUGGING
-// #define DEBUGGING
-
 html_text::html_text (simple_output *op, html_dialect d) :
   stackptr(NULL), lastptr(NULL), out(op), dialect(d),
   space_emitted(TRUE), current_indentation(-1),
@@ -215,8 +211,8 @@ void html_text::issue_tag (const char *tagname, const char *arg,
 
 void html_text::issue_color_begin (color *c)
 {
+  char buf[(INT_HEXDIGITS * 3) + 1];
   unsigned int r, g, b;
-  char buf[6+1];
 
   out->put_string("<font color=\"#");
   if (c->is_default())
@@ -756,7 +752,7 @@ html_indent *html_text::remove_indent (HTML_TAG tag)
 
 /*
  *  remove_para_space - removes the leading space to a paragraph
- *                      (effectively this trims off a leading `.sp' tag).
+ *                      (effectively this trims off a leading '.sp' tag).
  */
 
 void html_text::remove_para_space (void)
