@@ -1,25 +1,21 @@
-// -*- C++ -*-
-/* Copyright (C) 1994, 2000, 2001, 2002, 2003, 2004
-   Free Software Foundation, Inc.
-     Written by Francisco Andrs Verd <pandres@dragonet.es> with many ideas
-     taken from the other groff drivers.
-
+/* Copyright (C) 1994-2020 Free Software Foundation, Inc.
+     Written by Francisco Andrs Verd <pandres@dragonet.es> with many
+     ideas taken from the other groff drivers.
 
 This file is part of groff.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 groff is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
-You should have received a copy of the GNU General Public License along
-with groff; see the file COPYING.  If not, write to the Free Software
-Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 /*
 TODO
@@ -27,6 +23,8 @@ TODO
  - Add X command to include bitmaps
 */
 
+#include <assert.h>
+
 #include "driver.h"
 #include "lbp.h"
 #include "charset.h"
@@ -36,11 +34,14 @@ TODO
 
 extern "C" const char *Version_string;
 
-static int user_papersize = -1;		// papersize
-static int orientation = -1;		// orientation
-static double user_paperlength = 0;	// Custom Paper size
+static int user_papersize = -1;
+static int orientation = -1;
+
+// custom paper format
+static double user_paperlength = 0;
 static double user_paperwidth = 0;
-static int ncopies = 1;			// Number of copies
+
+static int ncopies = 1;
 
 #define DEFAULT_LINEWIDTH_FACTOR 40	// 0.04em
 static int linewidth_factor = DEFAULT_LINEWIDTH_FACTOR;
@@ -63,7 +64,7 @@ class lbp_printer : public printer {
 public:
   lbp_printer(int, double, double);
   ~lbp_printer();
-  void set_char(int, font *, const environment *, int, const char *name);
+  void set_char(glyph *, font *, const environment *, int, const char *name);
   void draw(int code, int *p, int np, const environment *env);
   void begin_page(int);
   void end_page(int page_length);
@@ -86,8 +87,9 @@ private:
   unsigned short cur_symbol_set;
   int line_thickness;
   int req_linethickness; // requested line thickness
+  // custom paper format
   int papersize;
-  int paperlength;	// custom paper size
+  int paperlength;
   int paperwidth;
 };
 
@@ -120,7 +122,7 @@ void lbp_font::handle_unknown_font_command(const char *command,
   if (strcmp(command, "lbpname") == 0) {
     if (arg == 0)
       fatal_with_file_and_line(filename, lineno,
-			       "`%1' command requires an argument",
+			       "'%1' command requires an argument",
 			       command);
     this->lbpname = new char[strlen(arg) + 1];
     strcpy(this->lbpname, arg);
@@ -157,10 +159,10 @@ lbp_printer::lbp_printer(int ps, double pw, double pl)
   lbpputs("\033c\033;\033[2&z\033[7 I\033[?32h\033[?33h\033[11h");
   wp54charset(); // Define the new symbol set
   lbpputs("\033[7 I\033[?32h\033[?33h\033[11h");
-  // Paper size handling
+  // Paper format handling
   if (orientation < 0)
     orientation = 0;	// Default orientation is portrait
-  papersize = 14;	// Default paper size is A4
+  papersize = 14;	// Default paper format is A4
   if (font::papersize) {
     papersize = set_papersize(font::papersize);
     paperlength = font::paperlength;
@@ -214,7 +216,7 @@ inline void lbp_printer::set_line_thickness(int size,const environment *env)
       /*  fprintf(stderr, "thickness: %d == %d, size %d, %d \n",
         size, line_thickness, env->size,req_linethickness); */
    return;
-}; // lbp_printer::set_line_thickness
+} // lbp_printer::set_line_thickness
 
 void lbp_printer::begin_page(int)
 {
@@ -277,10 +279,10 @@ char *lbp_printer::font_name(const lbp_font *f, const int siz)
   return bfont_name;
 }
 
-void lbp_printer::set_char(int idx, font *f, const environment *env,
+void lbp_printer::set_char(glyph *g, font *f, const environment *env,
 			   int w, const char *)
 {
-  int code = f->get_code(idx);
+  int code = f->get_code(g);
   unsigned char ch = code & 0xff;
   unsigned short symbol_set = code >> 8;
   if (f != cur_font) {
@@ -391,9 +393,10 @@ inline void lbp_printer::setfillmode(int mode)
     if (mode != 1)
       vdmsetfillmode(mode, 1, 0);
     else
-      vdmsetfillmode(mode, 1, 1);	// To get black we must use white
-					// inverted
-      fill_mode = mode;
+      // To get black, we must use white inverted.
+      vdmsetfillmode(mode, 1, 1);
+
+    fill_mode = mode;
   }
 }
 
@@ -403,7 +406,7 @@ inline void lbp_printer::polygon(int hpos, int vpos, int np, int *p)
   points = new int[np + 2];
   points[0] = hpos;
   points[1] = vpos;
-  // fprintf(stderr, "Poligon (%d,%d) ", points[0], points[1]);
+  // fprintf(stderr, "Polygon (%d,%d) ", points[0], points[1]);
   for (i = 0; i < np; i++)
     points[i + 2] = p[i];
   // for (i = 0; i < np; i++) fprintf(stderr, " %d ", p[i]);
@@ -424,9 +427,9 @@ void lbp_printer::draw(int code, int *p, int np, const environment *env)
       if (np != 1 && np != 2) {
 	error("0 or 1 argument required for thickness");
 	break;
-      };
+      }
     set_line_thickness(p[0],env);
-    };
+    }
     break;
   case 'l':	// Line
     if (np != 2) {
@@ -543,7 +546,7 @@ void lbp_printer::draw(int code, int *p, int np, const environment *env)
     // not implemented yet
     break;
   default:
-    error("unrecognised drawing command `%1'", char(code));
+    error("unrecognised drawing command '%1'", char(code));
     break;
   }
   return;
@@ -572,15 +575,15 @@ static struct {
 static int set_papersize(const char *paperformat)
 {
   unsigned int i;
-  // First test for a standard (i.e. supported directly by the printer)
-  // paper size
+  // First, test for a standard (i.e. supported directly by the printer)
+  // paper format.
   for (i = 0 ; i < sizeof(lbp_papersizes) / sizeof(lbp_papersizes[0]); i++)
   {
     if (strcasecmp(lbp_papersizes[i].name,paperformat) == 0)
       return lbp_papersizes[i].code;
   }
-  // Otherwise, we assume a custom paper size
-  return 82;
+  // Otherwise, we assume a custom paper format.
+  return 82; // XXX: magic number
 }
 
 static void handle_unknown_desc_command(const char *command, const char *arg,
@@ -588,12 +591,12 @@ static void handle_unknown_desc_command(const char *command, const char *arg,
 {
   // orientation command
   if (strcasecmp(command, "orientation") == 0) {
-    // We give priority to command line options
+    // We give priority to command-line options
     if (orientation > 0)
       return;
     if (arg == 0)
       error_with_file_and_line(filename, lineno,
-			       "`orientation' command requires an argument");
+			       "'orientation' command requires an argument");
     else {
       if (strcasecmp(arg, "portrait") == 0)
 	orientation = 0;
@@ -602,7 +605,7 @@ static void handle_unknown_desc_command(const char *command, const char *arg,
 	  orientation = 1;
 	else
 	  error_with_file_and_line(filename, lineno,
-				   "invalid argument to `orientation' command");
+				   "invalid argument to 'orientation' command");
       }
     }
   }
@@ -623,18 +626,20 @@ static struct option long_options[] = {
 static void usage(FILE *stream)
 {
   fprintf(stream,
-	  "usage: %s [-lvh] [-c n] [-p paper_size] [-F dir] [-o or]\n"
-	  "       [-w width] [files ...]\n"
-	  "\n"
-	  "  -o --orientation=[portrait|landscape]\n"
-	  "  -v --version\n"
-	  "  -c --copies=numcopies\n"
-	  "  -l --landscape\n"
-	  "  -p --papersize=paper_size\n"
-	  "  -w --linewidth=width\n"
-	  "  -F --fontdir=dir\n"
-	  "  -h --help\n",
-	  program_name);
+"usage: %s [-l] [-c num-copies] [-F font-directory] [-o orientation]"
+" [-p paper-format] [-w width] [file ...]\n"
+"usage: %s {-v | --version}\n"
+"usage: %s {-h | --help}\n",
+	  program_name, program_name, program_name);
+  if (stdout == stream) {
+    fputs(
+"\n"
+"Translate the output of troff(1) into a CaPSL and VDM format suitable"
+"\n"
+"for Canon LBP-4 and LBP-8 printers.  See the grolbp(1) manual page.\n",
+	  stream);
+    exit(EXIT_SUCCESS);
+  }
 }
 
 int main(int argc, char **argv)
@@ -643,11 +648,11 @@ int main(int argc, char **argv)
     program_name = strsave(argv[0]);
   font::set_unknown_desc_command_handler(handle_unknown_desc_command);
   // command line parsing
-  int c = 0;
+  int c;
   int option_index = 0;
-  while (c >= 0) {
-    c = getopt_long (argc, argv, "c:F:hI:lo:p:vw:",
-		     long_options, &option_index);
+  while ((c = getopt_long(argc, argv, "c:F:hI:lo:p:vw:", long_options,
+			  &option_index))
+	 != EOF) {
     switch (c) {
     case 'F':
       font::command_line_font_dir(optarg);
@@ -660,7 +665,7 @@ int main(int argc, char **argv)
 	const char *s;
 	if (!font::scan_papersize(optarg, &s,
 				  &user_paperlength, &user_paperwidth))
-	  error("invalid paper size `%1' ignored", optarg);
+	  error("ignoring invalid paper format '%1'", optarg);
 	else
 	  user_papersize = set_papersize(s);
 	break;
@@ -670,7 +675,7 @@ int main(int argc, char **argv)
       break;
     case 'v':
       printf("GNU grolbp (groff) version %s\n", Version_string);
-      exit(0);
+      exit(EXIT_SUCCESS);
       break;
     case 'o':
       if (strcasecmp(optarg, "portrait") == 0)
@@ -680,7 +685,7 @@ int main(int argc, char **argv)
 	  orientation = 1;
 	else
 	  error("unknown orientation '%1'", optarg);
-      };
+      }
       break;
     case 'c':
       {
@@ -708,18 +713,26 @@ int main(int argc, char **argv)
       }
     case 'h':
       usage(stdout);
-      exit(0);
       break;
     case '?':
       usage(stderr);
-      exit(1);
+      exit(EXIT_FAILURE);
       break;
+    default:
+      assert(0 == "unhandled getopt_long return value");
     }
   }
   if (optind >= argc)
     do_file("-");
   while (optind < argc)
     do_file(argv[optind++]);
-  lbpputs("\033c\033<");
+  if (lbpoutput)
+    lbpputs("\033c\033<");
   return 0;
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
