@@ -1,22 +1,21 @@
 // -*- C -*-
-/* Copyright (C) 1994, 2000, 2001, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1994-2020 Free Software Foundation, Inc.
      Written by Francisco Andrs Verd <pandres@dragonet.es>
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 groff is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
-You should have received a copy of the GNU General Public License along
-with groff; see the file COPYING.  If not, write to the Free Software
-Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
-/*  This file contains a set of utility functions to use canon CAPSL printers
+/*  This file contains a set of utility functions to use canon CaPSL printers
  *  (lbp-4 and lbp-8 series printers) */
 
 #ifndef LBP_H
@@ -28,14 +27,15 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 static FILE *lbpoutput = NULL;
 static FILE *vdmoutput = NULL;
 
+
 static inline void 
 lbpinit(FILE *outfile)
 {
 	lbpoutput = outfile;
-};
+}
 
 
-static inline void 
+static void 
 lbpprintf(const char *format, ... )
 { /* Taken from cjet */
   va_list stuff;
@@ -43,68 +43,78 @@ lbpprintf(const char *format, ... )
   va_start(stuff, format);
   vfprintf(lbpoutput, format, stuff);
   va_end(stuff);
-};
+}
+
 
 static inline void
 lbpputs(const char *data)
 {
 	fputs(data,lbpoutput);
-};
+}
+
 
 static inline void
 lbpputc(unsigned char c)
 {
 	fputc(c,lbpoutput);
-};
+}
 
 
 static inline void 
-lbpsavestatus(int index )
+lbpsavestatus(int idx )
 {
-	fprintf(lbpoutput,"\033[%d%%y",index);
-};
+	fprintf(lbpoutput,"\033[%d%%y",idx);
+}
+
 
 static inline void 
-lbprestorestatus(int index )
+lbprestorestatus(int idx )
 {
-	fprintf(lbpoutput,"\033[%d%cz",index ,'%');
-};
+	fprintf(lbpoutput,"\033[%d%cz",idx ,'%');
+}
+
 
 static inline void 
-lbpsavepos(int index)
+lbpsavepos(int idx)
 {
-	fprintf(lbpoutput,"\033[1;%d;0x",index);
-};
+	fprintf(lbpoutput,"\033[1;%d;0x",idx);
+}
+
 
 static inline void 
-lbprestorepos(int index)
+lbprestorepos(int idx)
 {
-	fprintf(lbpoutput,"\033[0;%d;0x",index);
-};
+	fprintf(lbpoutput,"\033[0;%d;0x",idx);
+}
+
 
 static inline void 
-lbprestoreposx(int index)
+lbprestoreposx(int idx)
 {
-	fprintf(lbpoutput,"\033[0;%d;1x",index);
-};
+	fprintf(lbpoutput,"\033[0;%d;1x",idx);
+}
+
 
 static inline void 
 lbpmoverel(int despl, char direction)
 {
 	fprintf(lbpoutput,"\033[%d%c",despl,direction);
-};
+}
+
 
 static inline void 
 lbplinerel(int width,int despl,char direction )
 {
 	fprintf(lbpoutput,"\033[%d;0;9{\033[%d%c\033[9}",width,despl,direction);
-};
+}
+
 
 static inline void 
 lbpmoveabs(int x, int y)
 {
 	fprintf(lbpoutput,"\033[%d;%df",y,x);
-};
+}
+
 
 static inline void
 lbplineto(int x,int y, int width )
@@ -112,7 +122,8 @@ lbplineto(int x,int y, int width )
 	fprintf(lbpoutput,"\033[%d;0;9{",width);
 	lbpmoveabs(x,y);
 	fprintf(lbpoutput,"\033[9}\n");
-};
+}
+
 
 static inline void
 lbpruleabs(int x, int y, int hsize, int vsize)
@@ -121,9 +132,11 @@ lbpruleabs(int x, int y, int hsize, int vsize)
 	fprintf(lbpoutput,"\033[0;9;000s");
 	lbpmoveabs(x+hsize,y+vsize);
 	fprintf(lbpoutput,"\033[9r");
-};
+}
+
+
+static void vdmprintf(const char *format, ... );
 
-static inline void vdmprintf(const char *format, ... );
 
 static inline char *
 vdmnum(int num,char *result)
@@ -147,7 +160,8 @@ vdmnum(int num,char *result)
   *p++ = b3;
   *p = 0x00; /* End of the resulting string */
   return result;
-};
+}
+
 
 static inline void
 vdmorigin(int newx, int newy)
@@ -155,7 +169,7 @@ vdmorigin(int newx, int newy)
    char nx[4],ny[4];
 
 	vdmprintf("}\"%s%s\x1e",vdmnum(newx,nx),vdmnum(newy,ny));
-}; /* vdmorigin */
+}
 
 
 static inline FILE *
@@ -170,15 +184,17 @@ vdminit(FILE *vdmfile)
 		  vdmnum(-3,scale),vdmnum(1,size),vdmnum(1,lineend));
   return vdmoutput;
   
-};
+}
+
 
 static inline void
 vdmend()
 {
 	vdmprintf("}p\x1e");
-};
+}
 
-static inline void 
+
+static void 
 vdmprintf(const char *format, ... )
 { /* Taken from cjet */
   va_list stuff;
@@ -187,7 +203,8 @@ vdmprintf(const char *format, ... )
   va_start(stuff, format);
   vfprintf(vdmoutput, format, stuff);
   va_end(stuff);
-};
+}
+
 
 static inline void
 vdmsetfillmode(int pattern,int perimeter, int inverted)
@@ -200,7 +217,8 @@ vdmsetfillmode(int pattern,int perimeter, int inverted)
    	vdmprintf("I%s%s%s%s%s\x1e",vdmnum(pattern,patt),\
 		vdmnum(perimeter,perim),vdmnum(0,rot),
 		vdmnum(0,espejo),vdmnum(inverted,inv));
-};
+}
+
 
 static inline void
 vdmcircle(int centerx, int centery, int radius)
@@ -209,7 +227,8 @@ vdmcircle(int centerx, int centery, int radius)
   
 	vdmprintf("5%s%s%s\x1e",vdmnum(centerx,x),vdmnum(centery,y),\
 			vdmnum(radius,rad));
-};
+}
+
 
 static inline void
 vdmaarc(int centerx, int centery, int radius,int startangle,int angle,int style,int arcopen)
@@ -220,7 +239,8 @@ vdmaarc(int centerx, int centery, int radius,int startangle,int angle,int style,
 		vdmnum(centerx,x),vdmnum(centery,y),\
 		vdmnum(radius,rad),vdmnum(startangle,stx),vdmnum(angle,sty),\
 		vdmnum(style,styl));
-};
+}
+
 
 static inline void
 vdmvarc(int centerx, int centery,int radius, int startx, int starty, int endx, int endy,\
@@ -228,11 +248,12 @@ vdmvarc(int centerx, int centery,int radius, int startx, int starty, int endx, i
 {
   char x[4],y[4],rad[4],stx[4],sty[4],enx[4],eny[4],styl[4],op[4];
   
-	vdmprintf("}6%s%s%s%s%s%s%s%s\x1e",vdmnum(arcopen,op),\
+	vdmprintf("}6%s%s%s%s%s%s%s%s%s\x1e",vdmnum(arcopen,op),\
 		vdmnum(centerx,x),vdmnum(centery,y),\
 		vdmnum(radius,rad),vdmnum(startx,stx),vdmnum(starty,sty),\
 		vdmnum(endx,enx),vdmnum(endy,eny),vdmnum(style,styl));
-};
+}
+
 
 static inline void
 vdmellipse(int centerx, int centery, int radiusx, int radiusy,int rotation)
@@ -242,7 +263,8 @@ vdmellipse(int centerx, int centery, int radiusx, int radiusy,int rotation)
 	vdmprintf("}7%s%s%s%s%s\x1e\n",vdmnum(centerx,x),vdmnum(centery,y),\
 			vdmnum(radiusx,radx),vdmnum(radiusy,rady),\
 			vdmnum(rotation,rotat));
-};
+}
+
 
 static inline void
 vdmsetlinetype(int lintype)
@@ -251,7 +273,8 @@ vdmsetlinetype(int lintype)
 
   vdmprintf("E1%s%s\x1e",vdmnum(lintype,ltyp),vdmnum(1,expfact));
   
-};
+}
+
 
 static inline void
 vdmsetlinestyle(int lintype, int pattern,int unionstyle)
@@ -265,7 +288,8 @@ vdmsetlinestyle(int lintype, int pattern,int unionstyle)
 		vdmnum(pattern,patt),vdmnum(0,rot),
 		vdmnum(0,espejo),vdmnum(0,in));
 	vdmprintf("}F%s",vdmnum(unionstyle,rot));
-};
+}
+
 
 static inline void
 vdmlinewidth(int width)
@@ -273,7 +297,8 @@ vdmlinewidth(int width)
   char wh[4];
 
   	vdmprintf("F1%s\x1e",vdmnum(width,wh));
-};
+}
+
 
 static inline void
 vdmrectangle(int origx, int origy,int dstx, int dsty)
@@ -282,7 +307,8 @@ vdmrectangle(int origx, int origy,int dstx, int dsty)
 
   vdmprintf("}:%s%s%s%s\x1e\n",vdmnum(origx,xcoord),vdmnum(dstx,sdstx),\
 		  vdmnum(origy,ycoord),vdmnum(dsty,sdsty));
-}; /* polyline */
+}
+
 
 static inline void
 vdmpolyline(int numpoints, int *points)
@@ -296,10 +322,11 @@ vdmpolyline(int numpoints, int *points)
   for (i = 1; i < numpoints ; i++) {
 	  vdmprintf("%s%s",vdmnum(*p,xcoord),vdmnum(*(p+1),ycoord));
 	  p += 2;
-  }; /* for */
+  } /* for */
   vdmprintf("\x1e\n");
-}; /* polyline */
+}
 	 
+
 static inline void
 vdmpolygon(int numpoints, int *points)
 {
@@ -312,20 +339,20 @@ vdmpolygon(int numpoints, int *points)
   for (i = 1; i < numpoints ; i++) {
 	  vdmprintf("%s%s",vdmnum(*p,xcoord),vdmnum(*(p+1),ycoord));
 	  p += 2;
-  }; /* for */
+  } /* for */
   vdmprintf("\x1e\n");
 
-}; /* vdmpolygon */
+}
 
 
 /************************************************************************
- *		Highter level auxiliary functions			*
- ************************************************************************/		
+ *		Higher level auxiliary functions			*
+ ************************************************************************/
 static inline int
 vdminited()
 {
 	return (vdmoutput != NULL);
-}; /* vdminited */
+}
 
 
 static inline void
@@ -340,7 +367,8 @@ vdmline(int startx, int starty, int sizex, int sizey)
 
   vdmpolyline(2,points);
 
-};
+}
+
 
 /*#define         THRESHOLD       .05    */ /* inch */
 #define         THRESHOLD       1     /* points (1/300 inch) */
@@ -348,13 +376,13 @@ static inline void
 splinerel(double px,double py,int flush)
 {
   static int lx = 0 ,ly = 0;
-  static float pend = 0.0;
+  static double pend = 0.0;
   static int dy = 0, despx = 0, despy = 0, sigpend = 0;
-  int dxnew ,dynew, sg;
+  int dxnew = 0, dynew = 0, sg;
   char xcoord[4],ycoord[4];
-  float npend ;
+  double npend ;
 
-  if (flush == -1) {lx = (int)px; ly = (int)py; return;};
+  if (flush == -1) {lx = (int)px; ly = (int)py; return;}
 
   if (flush == 0) {
   dxnew = (int)px -lx;
@@ -366,7 +394,7 @@ splinerel(double px,double py,int flush)
 	  despx = dxnew; 
 	  if ((sg == sigpend) && (dy == 0)){
 		  return;
-	  };
+	  }
 	dy = 0;
   }
   else {
@@ -377,9 +405,9 @@ splinerel(double px,double py,int flush)
   	else
   	{ sigpend = sg;
     	pend = npend;
-  	}; /* else (( npend == pend) && ... */
-  }; /* else (if (dynew == 0)) */
-  }; /* if (!flush ) */
+  	} /* else (( npend == pend) && ... */
+  } /* else (if (dynew == 0)) */
+  } /* if (!flush ) */
 
   /* if we've changed direction we must draw the line */
 /*  fprintf(stderr," (%d) %.2f,%.2f\n",flush,(float)px,(float)py);*/
@@ -390,7 +418,7 @@ splinerel(double px,double py,int flush)
   if (flush) {
   	dxnew = dy = despx = despy = 0;
 	return;
-  }; /* if (flush) */
+  } /* if (flush) */
   dxnew -= despx;
   dynew -= despy;
   if ((dxnew != 0) || (dynew != 0)) vdmprintf("%s%s",vdmnum(dxnew,xcoord),\
@@ -401,111 +429,116 @@ splinerel(double px,double py,int flush)
   lx = (int)px; ly = (int)py; 
   dxnew = dy = despx = despy = 0;
   
-}; /* splinerel */
+}
+
 
 /**********************************************************************
  *  The following code to draw splines is adapted from the transfig package
  */
 static void
-quadratic_spline(double a1,double  b1, double a2, double b2, \
-		double a3, double b3, double a4, double b4)
+quadratic_spline(double a_1, double b_1, double a_2, double b_2, \
+		 double a_3, double b_3, double a_4, double b_4)
 {
-	double	x1, y1, x4, y4;
-	double	xmid, ymid;
+	double	x_1, y_1, x_4, y_4;
+	double	x_mid, y_mid;
 
-	x1	 = a1; y1 = b1;
-	x4	 = a4; y4 = b4;
-	xmid	 = (a2 + a3)/2.0;
-	ymid	 = (b2 + b3)/2.0;
-	if ((fabs(x1 - xmid) < THRESHOLD) && (fabs(y1 - ymid) < THRESHOLD)) {
-        	splinerel(xmid,ymid,0);
-/*	    fprintf(tfp, "PA%.4f,%.4f;\n", xmid, ymid);*/
+	x_1	 = a_1; y_1 = b_1;
+	x_4	 = a_4; y_4 = b_4;
+	x_mid	 = (a_2 + a_3)/2.0;
+	y_mid	 = (b_2 + b_3)/2.0;
+	if ((fabs(x_1 - x_mid) < THRESHOLD)
+	    && (fabs(y_1 - y_mid) < THRESHOLD)) {
+        	splinerel(x_mid, y_mid, 0);
+/*	    fprintf(tfp, "PA%.4f,%.4f;\n", x_mid, y_mid);*/
 	}
 	else {
-	    quadratic_spline(x1, y1, ((x1+a2)/2.0), ((y1+b2)/2.0),
-		((3.0*a2+a3)/4.0), ((3.0*b2+b3)/4.0), xmid, ymid);
-	    }
+	    quadratic_spline(x_1, y_1, ((x_1+a_2)/2.0), ((y_1+b_2)/2.0),
+		((3.0*a_2+a_3)/4.0), ((3.0*b_2+b_3)/4.0), x_mid, y_mid);
+	}
 
-	if ((fabs(xmid - x4) < THRESHOLD) && (fabs(ymid - y4) < THRESHOLD)) {
-		splinerel(x4,y4,0);
-/*	    fprintf(tfp, "PA%.4f,%.4f;\n", x4, y4);*/
+	if ((fabs(x_mid - x_4) < THRESHOLD)
+	    && (fabs(y_mid - y_4) < THRESHOLD)) {
+		splinerel(x_4, y_4, 0);
+/*	    fprintf(tfp, "PA%.4f,%.4f;\n", x_4, y_4);*/
 	}
 	else {
-	    quadratic_spline(xmid, ymid, ((a2+3.0*a3)/4.0), ((b2+3.0*b3)/4.0),
-			((a3+x4)/2.0), ((b3+y4)/2.0), x4, y4);
-	    };
-}; /* quadratic_spline */
+	    quadratic_spline(x_mid, y_mid,
+			     ((a_2+3.0*a_3)/4.0), ((b_2+3.0*b_3)/4.0),
+			     ((a_3+x_4)/2.0), ((b_3+y_4)/2.0), x_4, y_4);
+	}
+}
+
 
 #define XCOORD(i) numbers[(2*i)]
 #define YCOORD(i) numbers[(2*i)+1]
 static void
-vdmspline(int numpoints, int ox,int oy, int *numbers)
+vdmspline(int numpoints, int o_x, int o_y, int *numbers)
 {
-	double	cx1, cy1, cx2, cy2, cx3, cy3, cx4, cy4;
-	double	x1, y1, x2, y2;
+	double	cx_1, cy_1, cx_2, cy_2, cx_3, cy_3, cx_4, cy_4;
+	double	x_1, y_1, x_2, y_2;
 	char xcoord[4],ycoord[4];
 	int i;
 
 	/*p	 = s->points;
-	x1	 = p->x/ppi;*/
-	x1	 = ox;
-	y1	 = oy;
+	x_1	 = p->x/ppi;*/
+	x_1	 = o_x;
+	y_1	 = o_y;
 /*	p	 = p->next;
-	x2	 = p->x/ppi;
-	y2	 = p->y/ppi;*/
-	x2	 = ox + XCOORD(0);
-	y2	 = oy + YCOORD(0);
-	cx1	 = (x1 + x2)/2.0;
-	cy1	 = (y1 + y2)/2.0;
-	cx2	 = (x1 + 3.0*x2)/4.0;
-	cy2	 = (y1 + 3.0*y2)/4.0;
+	x_2	 = p->x/ppi;
+	y_2	 = p->y/ppi;*/
+	x_2	 = o_x + XCOORD(0);
+	y_2	 = o_y + YCOORD(0);
+	cx_1	 = (x_1 + x_2)/2.0;
+	cy_1	 = (y_1 + y_2)/2.0;
+	cx_2	 = (x_1 + 3.0*x_2)/4.0;
+	cy_2	 = (y_1 + 3.0*y_2)/4.0;
 
-/*	fprintf(stderr,"Spline %d (%d,%d)\n",numpoints,(int)x1,(int)y1);*/
-    	vdmprintf("1%s%s",vdmnum((int)x1,xcoord),vdmnum((int)y1,ycoord));
-	splinerel(x1,y1,-1);
-	splinerel(cx1,cy1,0);
+/*	fprintf(stderr,"Spline %d (%d,%d)\n",numpoints,(int)x_1,(int)y_1);*/
+    	vdmprintf("1%s%s",vdmnum((int)x_1,xcoord),vdmnum((int)y_1,ycoord));
+	splinerel(x_1,y_1,-1);
+	splinerel(cx_1,cy_1,0);
 /*	    fprintf(tfp, "PA%.4f,%.4f;PD%.4f,%.4f;\n",
-		    x1, y1, cx1, cy1);*/
+		    x_1, y_1, cx_1, cy_1);*/
 
 	/*for (p = p->next; p != NULL; p = p->next) {*/
 	for (i = 1; i < (numpoints); i++) {
-	    x1	 = x2;
-	    y1	 = y2;
-/*	    x2	 = p->x/ppi;
-	    y2	 = p->y/ppi;*/
-            x2   = x1 + XCOORD(i);
-            y2   = y1 + YCOORD(i);
-	    cx3	 = (3.0*x1 + x2)/4.0;
-	    cy3	 = (3.0*y1 + y2)/4.0;
-	    cx4	 = (x1 + x2)/2.0;
-	    cy4	 = (y1 + y2)/2.0;
-	    /* fprintf(stderr,"Point (%d,%d) - (%d,%d)\n",(int)x1,(int)(y1),(int)x2,(int)y2);*/
-	    quadratic_spline(cx1, cy1, cx2, cy2, cx3, cy3, cx4, cy4);
-	    cx1	 = cx4;
-	    cy1	 = cy4;
-	    cx2	 = (x1 + 3.0*x2)/4.0;
-	    cy2	 = (y1 + 3.0*y2)/4.0;
-	    }
-	x1	 = x2; 
-	y1	 = y2;
-/*	p	 = s->points->next;
-	x2	 = p->x/ppi;
-	y2	 = p->y/ppi;*/
-        x2       = ox + XCOORD(0);
-        y2       = oy + YCOORD(0);
-	cx3	 = (3.0*x1 + x2)/4.0;
-	cy3	 = (3.0*y1 + y2)/4.0;
-	cx4	 = (x1 + x2)/2.0;
-	cy4	 = (y1 + y2)/2.0;
-	splinerel(x1,y1,0);
-	splinerel(x1,y1,1);
-       	/*vdmprintf("%s%s",vdmnum((int)(x1-lx),xcoord),\
-			vdmnum((int)(y1-ly),ycoord));*/
+	    x_1	 = x_2;
+	    y_1	 = y_2;
+/*	    x_2	 = p->x/ppi;
+	    y_2	 = p->y/ppi;*/
+            x_2	 = x_1 + XCOORD(i);
+            y_2	 = y_1 + YCOORD(i);
+	    cx_3 = (3.0*x_1 + x_2)/4.0;
+	    cy_3 = (3.0*y_1 + y_2)/4.0;
+	    cx_4 = (x_1 + x_2)/2.0;
+	    cy_4 = (y_1 + y_2)/2.0;
+	    /* fprintf(stderr,"Point (%d,%d) - (%d,%d)\n",(int)x_1,(int)(y_1),(int)x_2,(int)y_2);*/
+	    quadratic_spline(cx_1, cy_1, cx_2, cy_2, cx_3, cy_3, cx_4, cy_4);
+	    cx_1 = cx_4;
+	    cy_1 = cy_4;
+	    cx_2 = (x_1 + 3.0*x_2)/4.0;
+	    cy_2 = (y_1 + 3.0*y_2)/4.0;
+	}
+	x_1	= x_2; 
+	y_1	= y_2;
+/*	p	= s->points->next;
+	x_2	= p->x/ppi;
+	y_2	= p->y/ppi;*/
+        x_2     = o_x + XCOORD(0);
+        y_2     = o_y + YCOORD(0);
+	cx_3	= (3.0*x_1 + x_2)/4.0;
+	cy_3	= (3.0*y_1 + y_2)/4.0;
+	cx_4	= (x_1 + x_2)/2.0;
+	cy_4	= (y_1 + y_2)/2.0;
+	splinerel(x_1, y_1, 0);
+	splinerel(x_1, y_1, 1);
+       	/*vdmprintf("%s%s",vdmnum((int)(x_1-lx),xcoord),\
+			vdmnum((int)(y_1-ly),ycoord));*/
         vdmprintf("\x1e\n");
-/*	    fprintf(tfp, "PA%.4f,%.4f;PU;\n", x1, y1);*/
+/*	    fprintf(tfp, "PA%.4f,%.4f;PU;\n", x_1, y_1);*/
 
 
-}; /* vdmspline */
+}
 	
 
 #endif
