@@ -4,17 +4,21 @@
  * map dvi fonts to X fonts
  */
 
+#include <config.h>
+
 #include <X11/Xos.h>
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
+
 #include "DviP.h"
 #include "XFontName.h"
+#include "font.h"
 
+/* forward reference */
 static void DisposeFontSizes(DviWidget, DviFontSizeList *);
-void DestroyFontMap(DviFontMap *);
 
 static char *
 savestr (const char *s)
@@ -135,7 +139,7 @@ ConvertFontNameToEncoding (char *n)
 	return n;
 }
 
-DviFontSizeList *
+static DviFontSizeList *
 InstallFontSizes (DviWidget dw, const char *x_name, Boolean *scalablep)
 {
 	char	fontNameString[2048];
