@@ -50,11 +50,13 @@ static char rcsid[] = "$XConsortium: xditview.c,v 1.17 89/12/10 17:05:08 rws Exp
 #include <X11/Xaw/SimpleMenu.h>
 #include <X11/Xaw/SmeBSB.h>
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <stdio.h>
 
 #include "Dvi.h"
+#include "groff_version.h"
 
 #include "xdit.bm"
 #include "xdit_mask.bm"
@@ -93,7 +95,7 @@ static XtResource resources[] = {
 #undef offset
 
 /* Command line options table.  Only resources are entered here...there is a
-   pass over the remaining options after XtParseCommand is let loose. */
+   pass over the remaining options after XrmParseCommand is let loose. */
 
 static XrmOptionDescRec options[] = {
 {(char *)"-page", (char *)"*dvi.pageNumber",
@@ -120,15 +122,30 @@ static FILE	*current_file;
  */
 
 static void
-Syntax(const char *call)
+Syntax(const char *progname, bool had_error)
 {
-	(void) printf ("Usage: %s [-fg <color>] [-bg <color>]\n", call);
-	(void) printf ("       [-bd <color>] [-bw <pixels>] [-help]\n");
-	(void) printf ("       [-display displayname] [-geometry geom]\n");
-	(void) printf ("       [-page <page-number>] [-backing <backing-store>]\n");
-	(void) printf ("       [-resolution <res>] [-print <command>]\n");
-	(void) printf ("       [-filename <file>] [filename]\n\n");
-	exit(1);
+	FILE *stream = stdout;
+	if (had_error)
+		stream = stderr;
+	(void) fprintf (stream, "usage: %s [X-toolkit-option]"
+			" [-backingStore backing-store-type]"
+			" [-filename file]"
+// See draw.c:FlushCharCache().
+#if 0
+			" [-noPolyText]"
+#endif
+			" [-page page-number]"
+			" [-printCommand command]"
+			" [-resolution resolution]"
+			" [file]\n", progname);
+	(void) fprintf (stream, "usage: %s {-version | --version}\n",
+			progname);
+	(void) fprintf (stream, "usage: %s {-help | --help}\n",
+			progname);
+	if (had_error)
+		exit(EXIT_FAILURE);
+	else
+		exit(EXIT_SUCCESS);
 }
 
 static void	NewFile (const char *);
@@ -205,10 +222,19 @@ int main(int argc, char **argv)
     toplevel = XtAppInitialize(&xtcontext, "GXditview",
 			    options, XtNumber (options),
  			    &argc, argv, fallback_resources, NULL, 0);
-    if (argc > 2
-	|| (argc == 2 && (!strcmp(argv[1], "-help")
-			  || !strcmp(argv[1], "--help"))))
-	Syntax(argv[0]);
+    if (argc > 2)
+	Syntax(argv[0], true /* had error */);
+    else if (argc == 2) {
+	if ((strcmp(argv[1], "-help") == 0)
+	    || (strcmp(argv[1], "--help") == 0))
+		Syntax(argv[0], false /* did not have error */);
+	else if ((strcmp(argv[1], "-version") == 0)
+	    || (strcmp(argv[1], "--version") == 0)) {
+		(void) printf("GNU gxditview (groff) version %s\n",
+			      Version_string);
+		exit(EXIT_SUCCESS);
+	}
+    }
 
     XtGetApplicationResources(toplevel, (XtPointer)&app_resources,
 			      resources, XtNumber(resources),
@@ -444,7 +470,7 @@ DoPrint (const char *name)
     FILE *print_file;
     RETSIGTYPE (*handler)(int);
 
-    /* Avoid dieing because of an invalid command. */
+    /* Avoid dying because of an invalid command. */
     handler = signal(SIGPIPE, SIG_IGN);
 
     print_file = popen(name, "w");
