@@ -1,5 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2018 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -19,14 +18,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <assert.h>
 
-// name2(a,b) concatenates two C identifiers.
-#ifdef TRADITIONAL_CPP
-# define name2(a,b) a/**/b
-#else /* not TRADITIONAL_CPP */
-# define name2(a,b) name2x(a,b)
-# define name2x(a,b) a ## b
-#endif /* not TRADITIONAL_CPP */
-
 // 'class ITABLE(T)' is the type of a hash table mapping an integer (int >= 0)
 // to an object of type T.
 //
@@ -38,9 +29,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 //
 // Nowadays one would use templates for this; this code predates the addition
 // of templates to C++.
-#define ITABLE(T) name2(T,_itable)
-#define IASSOC(T) name2(T,_iassoc)
-#define ITABLE_ITERATOR(T) name2(T,_itable_iterator)
+#define ITABLE(T) T ## _itable
+#define IASSOC(T) T ## _iassoc
+#define ITABLE_ITERATOR(T) T ## _itable_iterator
 
 // ptable.h declares this too
 #ifndef NEXT_PTABLE_SIZE_DEFINED
@@ -118,8 +109,8 @@ ITABLE(T)::ITABLE(T)()							      \
 ITABLE(T)::~ITABLE(T)()							      \
 {									      \
   for (unsigned i = 0; i < size; i++)					      \
-    a_delete v[i].val;							      \
-  a_delete v;								      \
+    delete[] v[i].val;							      \
+  delete[] v;								      \
 }									      \
 									      \
 void ITABLE(T)::define(int key, T *val)					      \
@@ -131,7 +122,7 @@ void ITABLE(T)::define(int key, T *val)					      \
        v[n].key >= 0;							      \
        n = (n == 0 ? size - 1 : n - 1))					      \
     if (v[n].key == key) {						      \
-      a_delete v[n].val;						      \
+      delete[] v[n].val;						      \
       v[n].val = val;							      \
       return;								      \
     }									      \
@@ -158,7 +149,7 @@ void ITABLE(T)::define(int key, T *val)					      \
 	 v[n].key >= 0;							      \
 	 n = (n == 0 ? size - 1 : n - 1))				      \
       ;									      \
-    a_delete oldv;							      \
+    delete[] oldv;							      \
   }									      \
   v[n].key = key;							      \
   v[n].val = val;							      \
@@ -195,4 +186,8 @@ int ITABLE_ITERATOR(T)::next(int *keyp, T **valp)			      \
   return 0;								      \
 }
 
-// end of itable.h
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
