@@ -1,5 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2018 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -17,6 +16,9 @@ for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
+#ifndef GROFF_LIB_H
+#define GROFF_LIB_H
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -25,14 +27,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 #define _ALL_SOURCE
 #endif
 
+#ifdef __cplusplus
 extern "C" {
+#endif
 #ifndef HAVE_STRERROR
   char *strerror(int);
 #endif
   const char *i_to_a(int);
   const char *ui_to_a(unsigned int);
   const char *if_to_a(int, int);
+#ifdef __cplusplus
 }
+#endif
 
 #define __GETOPT_PREFIX groff_
 #include <getopt.h>
@@ -47,8 +53,10 @@ extern "C" {
 #define getlocale(category) ((void)(category), (char *)"C")
 #endif /* !HAVE_SETLOCALE */
 
+#include <stdbool.h>
+
 char *strsave(const char *s);
-int is_prime(unsigned);
+bool is_prime(unsigned);
 double groff_hypot(double, double);
 
 #include <stdio.h>
@@ -59,9 +67,13 @@ double groff_hypot(double, double);
 
 #include <stdarg.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* LynxOS 4.0.0 doesn't declare vfprintf() */
 #ifdef NEED_DECLARATION_VFPRINTF
-extern "C" { int vfprintf(FILE *, const char *, va_list); }
+  int vfprintf(FILE *, const char *, va_list);
 #endif
 
 #ifndef HAVE_MKSTEMP
@@ -72,93 +84,78 @@ int mkstemp(char *tmpl);
 
 int mksdir(char *tmpl);
 
-FILE *xtmpfile(char **namep = 0,
-	       const char *postfix_long = 0, const char *postfix_short = 0,
-	       int do_unlink = 1);
-char *xtmptemplate(const char *postfix_long, const char *postfix_short);
+#ifdef __cplusplus
+  FILE *xtmpfile(char **namep = 0,
+		 const char *postfix_long = 0,
+		 const char *postfix_short = 0,
+		 int do_unlink = 1);
+  char *xtmptemplate(const char *postfix_long,
+		     const char *postfix_short);
+#endif
 
 #ifdef NEED_DECLARATION_POPEN
-extern "C" { FILE *popen(const char *, const char *); }
+  FILE *popen(const char *, const char *);
 #endif /* NEED_DECLARATION_POPEN */
 
 #ifdef NEED_DECLARATION_PCLOSE
-extern "C" { int pclose (FILE *); }
+  int pclose (FILE *);
 #endif /* NEED_DECLARATION_PCLOSE */
 
-size_t file_name_max(const char *fname);
-size_t path_name_max();
+  size_t file_name_max(const char *fname);
+  size_t path_name_max(void);
 
-extern char invalid_char_table[];
+  extern char invalid_char_table[];
 
-inline int invalid_input_char(int c)
-{
-  return c >= 0 && invalid_char_table[c];
-}
+  inline bool is_invalid_input_char(int c)
+  {
+    return (c >= 0 && invalid_char_table[c]);
+  }
 
 #ifdef HAVE_STRCASECMP
 #ifdef NEED_DECLARATION_STRCASECMP
 // Ultrix4.3's string.h fails to declare this.
-extern "C" { int strcasecmp(const char *, const char *); }
+  int strcasecmp(const char *, const char *); }
 #endif /* NEED_DECLARATION_STRCASECMP */
 #else /* !HAVE_STRCASECMP */
-extern "C" { int strcasecmp(const char *, const char *); }
+  int strcasecmp(const char *, const char *);
 #endif /* HAVE_STRCASECMP */
 
 #if !defined(_AIX) && !defined(sinix) && !defined(__sinix__)
 #ifdef HAVE_STRNCASECMP
 #ifdef NEED_DECLARATION_STRNCASECMP
 // SunOS's string.h fails to declare this.
-extern "C" { int strncasecmp(const char *, const char *, int); }
+  int strncasecmp(const char *, const char *, int);
 #endif /* NEED_DECLARATION_STRNCASECMP */
 #else /* !HAVE_STRNCASECMP */
-extern "C" { int strncasecmp(const char *, const char *, size_t); }
+  int strncasecmp(const char *, const char *, size_t);
 #endif /* HAVE_STRNCASECMP */
 #endif /* !_AIX && !sinix && !__sinix__ */
 
+#ifdef __cplusplus
+}
+#endif
+
 #ifdef HAVE_CC_LIMITS_H
 #include <limits.h>
 #else /* !HAVE_CC_LIMITS_H */
 #define INT_MAX 2147483647
 #endif /* !HAVE_CC_LIMITS_H */
 
-/* It's not safe to rely on people getting INT_MIN right (ie signed). */
-
-#ifdef INT_MIN
-#undef INT_MIN
-#endif
-
-#ifdef CFRONT_ANSI_BUG
-
-/* This works around a bug in cfront 2.0 used with ANSI C compilers. */
-
-#define INT_MIN ((long)(-INT_MAX-1))
-
-#else /* !CFRONT_ANSI_BUG */
-
-#define INT_MIN (-INT_MAX-1)
-
-#endif /* !CFRONT_ANSI_BUG */
-
-/* Maximum number of digits in the decimal representation of an int
-   (not including the -). */
-
-#define INT_DIGITS 10
+/* Maximum number of digits in decimal representations of `int` types
+   not including a leading minus sign. */
+#define INT_DIGITS 19		/* enough for 64 bit integer */
+#define UINT_DIGITS 20
 
 #ifdef PI
 #undef PI
 #endif
 
-const double PI = 3.14159265358979323846;
+static const double PI = 3.14159265358979323846;
 
-/* ad_delete deletes an array of objects with destructors;
-   a_delete deletes an array of objects without destructors */
+#endif /* GROFF_LIB_H */
 
-#ifdef ARRAY_DELETE_NEEDS_SIZE
-/* for 2.0 systems */
-#define ad_delete(size) delete [size]
-#define a_delete delete
-#else /* !ARRAY_DELETE_NEEDS_SIZE */
-/* for ARM systems */
-#define ad_delete(size) delete []
-#define a_delete delete []
-#endif /* !ARRAY_DELETE_NEEDS_SIZE */
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
