@@ -1,5 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2018 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -46,7 +45,7 @@ const
 #endif
 int minus_one = -1;
 
-int verify_flag = 0;
+bool do_verify = false;
 
 struct word_list;
 
@@ -68,7 +67,7 @@ class index_search_item : public search_item {
   const char *ignore_fields;
   time_t mtime;
 
-  const char *do_verify();
+  const char *get_invalidity_reason();
   const int *search1(const char **pp, const char *end);
   const int *search(const char *ptr, int length, int **temp_listp);
   const char *munge_filename(const char *);
@@ -77,9 +76,10 @@ class index_search_item : public search_item {
 public:
   index_search_item(const char *, int);
   ~index_search_item();
-  int load(int fd);
+  const char *check_header(index_header *, unsigned);
+  bool load(int fd);
   search_item_iterator *make_search_item_iterator(const char *);
-  int verify();
+  bool is_valid();
   void check_files();
   int next_filename_id() const;
   friend class index_search_item_iterator;
@@ -124,12 +124,12 @@ index_search_item::~index_search_item()
     out_of_date_files = out_of_date_files->next;
     delete tem;
   }
-  a_delete filename_buffer;
-  a_delete key_buffer;
+  delete[] filename_buffer;
+  delete[] key_buffer;
   if (common_words_table) {
     for (int i = 0; i < common_words_table_size; i++)
-      a_delete common_words_table[i];
-    a_delete common_words_table;
+      delete[] common_words_table[i];
+    delete[] common_words_table;
   }
 }
 
@@ -139,28 +139,70 @@ public:
   file_closer(int &fd) : fdp(&fd) { }
   ~file_closer() { close(*fdp); }
 };
- 
+
 // Tell the compiler that a variable is intentionally unused.
 inline void unused(void *) { }
 
-int index_search_item::load(int fd)
+// Validate the data reported in the header so that we don't overread on
+// the heap in the load() member function.  Return null pointer if no
+// problems are detected.
+const char *index_search_item::check_header(index_header *file_header,
+					    unsigned file_size)
+{
+  if (file_header->tags_size < 0)
+    return "tag list length negative";
+  if (file_header->lists_size < 0)
+    return "reference list length negative";
+  // The table and string pool sizes will not be zero, even in an empty
+  // index.
+  if (file_header->table_size < 1)
+    return "table size nonpositive";
+  if (file_header->strings_size < 1)
+    return "string pool size nonpositive";
+  size_t sz = (file_header->tags_size * sizeof(tag)
+	       + file_header->lists_size * sizeof(int)
+	       + file_header->table_size * sizeof(int)
+	       + file_header->strings_size
+	       + sizeof(file_header));
+  if (sz != file_size)
+    return("size mismatch between header and data");
+  unsigned size_remaining = file_size;
+  unsigned chunk_size = file_header->tags_size * sizeof(tag);
+  if (chunk_size > size_remaining)
+    return "claimed tag list length exceeds file size";
+  size_remaining -= chunk_size;
+  chunk_size = file_header->lists_size * sizeof(int);
+  if (chunk_size > size_remaining)
+    return "claimed reference list length exceeds file size";
+  size_remaining -= chunk_size;
+  chunk_size = file_header->table_size * sizeof(int);
+  if (chunk_size > size_remaining)
+    return "claimed table size exceeds file size";
+  size_remaining -= chunk_size;
+  chunk_size = file_header->strings_size;
+  if (chunk_size > size_remaining)
+    return "claimed string pool size exceeds file size";
+  return 0;
+}
+
+bool index_search_item::load(int fd)
 {
   file_closer fd_closer(fd);	// close fd on return
   unused(&fd_closer);
   struct stat sb;
   if (fstat(fd, &sb) < 0) {
-    error("can't fstat '%1': %2", name, strerror(errno));
-    return 0;
+    error("can't fstat index '%1': %2", name, strerror(errno));
+    return false;
   }
   if (!S_ISREG(sb.st_mode)) {
-    error("'%1' is not a regular file", name);
-    return 0;
+    error("index '%1' is not a regular file", name);
+    return false;
   }
   mtime = sb.st_mtime;
-  int size = int(sb.st_size);
+  unsigned size = unsigned(sb.st_size); // widening conversion
   if (size == 0) {
-    error("'%1' is an empty file", name);
-    return 0;
+    error("index '%1' is an empty file", name);
+    return false;
   }
   char *addr;
   map_addr = mapread(fd, size);
@@ -171,20 +213,20 @@ int index_search_item::load(int fd)
   else {
     addr = buffer = (char *)malloc(size);
     if (buffer == 0) {
-      error("can't allocate buffer for '%1'", name);
-      return 0;
+      error("can't allocate memory to process index '%1'", name);
+      return false;
     }
     char *ptr = buffer;
     int bytes_to_read = size;
     while (bytes_to_read > 0) {
       int nread = read(fd, ptr, bytes_to_read);
       if (nread == 0) {
-	error("unexpected EOF on '%1'", name);
-	return 0;
+	error("unexpected end-of-file while reading index '%1'", name);
+	return false;
       }
       if (nread < 0) {
-	error("read error on '%1': %2", name, strerror(errno));
-	return 0;
+	error("read error on index '%1': %2", name, strerror(errno));
+	return false;
       }
       bytes_to_read -= nread;
       ptr += nread;
@@ -193,22 +235,20 @@ int index_search_item::load(int fd)
   header = *(index_header *)addr;
   if (header.magic != INDEX_MAGIC) {
     error("'%1' is not an index file: wrong magic number", name);
-    return 0;
+    return false;
   }
   if (header.version != INDEX_VERSION) {
-    error("version number in '%1' is wrong: was %2, should be %3",
+    error("version number in index '%1' is wrong: was %2, should be %3",
 	  name, header.version, INDEX_VERSION);
-    return 0;
+    return false;
   }
-  int sz = (header.tags_size * sizeof(tag)
-	    + header.lists_size * sizeof(int)
-	    + header.table_size * sizeof(int)
-	    + header.strings_size
-	    + sizeof(header));
-  if (sz != size) {
-    error("size of '%1' is wrong: was %2, should be %3",
-	  name, size, sz);
-    return 0;
+  const char *problem = check_header(&header, size);
+  if (problem != 0) {
+    if (do_verify)
+      error("corrupt header in index file '%1': %2", name, problem);
+    else
+      error("corrupt header in index file '%1'", name);
+    return false;
   }
   tags = (tag *)(addr + sizeof(header));
   lists = (int *)(tags + header.tags_size);
@@ -217,14 +257,14 @@ int index_search_item::load(int fd)
   ignore_fields = strchr(strchr(pool, '\0') + 1, '\0') + 1;
   key_buffer = new char[header.truncate];
   read_common_words_file();
-  return 1;
+  return true;
 }
 
-const char *index_search_item::do_verify()
+const char *index_search_item::get_invalidity_reason()
 {
   if (tags == 0)
     return "not loaded";
-  if (lists[header.lists_size - 1] >= 0)
+  if ((header.lists_size > 0) && (lists[header.lists_size - 1] >= 0))
     return "last list element not negative";
   int i;
   for (i = 0; i < header.table_size; i++) {
@@ -249,17 +289,17 @@ const char *index_search_item::do_verify()
       return "bad start in tags";
   }
   if (pool[header.strings_size - 1] != '\0')
-    return "last character in pool not nul";
+    return "last character in string pool is not null";
   return 0;
 }
 
-int index_search_item::verify()
+bool index_search_item::is_valid()
 {
-  const char *reason = do_verify();
+  const char *reason = get_invalidity_reason();
   if (!reason)
-    return 1;
+    return true;
   error("'%1' is bad: %2", name, reason);
-  return 0;
+  return false;
 }
 
 int index_search_item::next_filename_id() const
@@ -282,13 +322,13 @@ search_item *make_index_search_item(const char *filename, int fid)
   if (fd < 0)
     return 0;
   index_search_item *item = new index_search_item(index_filename, fid);
-  a_delete index_filename;
+  delete[] index_filename;
   if (!item->load(fd)) {
     close(fd);
     delete item;
     return 0;
   }
-  else if (verify_flag && !item->verify()) {
+  else if (do_verify && !item->is_valid()) {
     delete item;
     return 0;
   }
@@ -316,9 +356,9 @@ index_search_item_iterator::index_search_item_iterator(index_search_item *ind,
 
 index_search_item_iterator::~index_search_item_iterator()
 {
-  a_delete temp_list;
-  a_delete buf;
-  a_delete query;
+  delete[] temp_list;
+  delete[] buf;
+  delete[] query;
   delete out_of_date_files_iter;
 }
 
@@ -405,7 +445,7 @@ int index_search_item_iterator::get_tag(int tagno,
     }
     if (!err) {
       if (length + 2 > buflen) {
-	a_delete buf;
+	delete[] buf;
 	buflen = length + 2;
 	buf = new char[buflen];
       }
@@ -448,7 +488,7 @@ const char *index_search_item::munge_filename(const char *filename)
 		    && strchr(DIR_SEPS, strchr(cwd, '\0')[-1]) == 0);
   int len = strlen(cwd) + strlen(filename) + need_slash + 1;
   if (len > filename_buflen) {
-    a_delete filename_buffer;
+    delete[] filename_buffer;
     filename_buflen = len;
     filename_buffer = new char[len];
   }
@@ -515,7 +555,7 @@ const int *index_search_item::search(const char *ptr, int length,
 {
   const char *end = ptr + length;
   if (*temp_listp) {
-    a_delete *temp_listp;
+    delete[] *temp_listp;
     *temp_listp = 0;
   }
   const int *first_list = 0;
@@ -546,12 +586,12 @@ const int *index_search_item::search(const char *ptr, int length,
     const int *list = search1(&ptr, end);
     if (list != 0) {
       if (*list < 0) {
-	a_delete matches;
+	delete[] matches;
 	return list;
       }
       merge(matches, matches, list);
       if (*matches < 0) {
-	a_delete matches;
+	delete[] matches;
 	return &minus_one;
       }
     }
@@ -573,7 +613,7 @@ void index_search_item::read_common_words_file()
   }
   common_words_table_size = 2*header.common + 1;
   while (!is_prime(common_words_table_size))
-    common_words_table_size++;
+    common_words_table_size += 2;
   common_words_table = new char *[common_words_table_size];
   for (int i = 0; i < common_words_table_size; i++)
     common_words_table[i] = 0;
@@ -618,7 +658,7 @@ void index_search_item::add_out_of_date_file(int fd, const char *filename,
     if ((*pp)->is_named(filename))
       return;
   *pp = make_linear_search_item(fd, filename, fid);
-  warning("'%1' modified since '%2' created", filename, name);
+  warning("'%1' modified since index '%2' created", filename, name);
 }
 
 void index_search_item::check_files()
@@ -640,3 +680,9 @@ void index_search_item::check_files()
     }
   }
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
