@@ -1,6 +1,5 @@
-// -*- C++ -*-
 /* Provide relocation for macro and font files.
-   Copyright (C) 2005-2018 Free Software Foundation, Inc.
+   Copyright (C) 2005-2020 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published
@@ -19,6 +18,7 @@
 
 #include "lib.h"
 
+#include <assert.h>
 #include <errno.h>
 #include <stdlib.h>
 
@@ -115,7 +115,7 @@ char *searchpath(const char *name, const char *pathp)
 #endif
       return path;
     }
-    a_delete path;
+    delete[] path;
     if (*end == '\0')
       break;
     p = end + 1;
@@ -135,12 +135,12 @@ char *searchpathext(const char *name, const char *pathext, const char *pathp)
     strcpy(namex, name);
     strcat(namex, ext);
     found = searchpath(namex, pathp);
-    a_delete namex;
+    delete[] namex;
     if (found)
        break;
     ext = strtok(0, PATH_SEP);
   }
-  a_delete tmpathext;
+  delete[] tmpathext;
   return found;
 }
 
@@ -159,27 +159,27 @@ char *msw2posixpath(char *path)
 // Compute the current prefix.
 void set_current_prefix()
 {
-  char *pathextstr;
-  curr_prefix = new char[path_name_max()];
   // Obtain the full path of the current binary;
   // using GetModuleFileName on MS-Windows,
   // and searching along PATH on other systems.
 #ifdef _WIN32
+  char *pathextstr;
+  curr_prefix = new char[path_name_max()];
   int len = GetModuleFileName(0, curr_prefix, path_name_max());
   if (len)
     len = GetShortPathName(curr_prefix, curr_prefix, path_name_max());
 # if DEBUG
   fprintf(stderr, "curr_prefix: %s\n", curr_prefix);
 # endif /* DEBUG */
-#else /* !_WIN32 */
-  curr_prefix = searchpath(program_name, getenv("PATH"));
   if (!curr_prefix && !strchr(program_name, '.')) {	// try with extensions
     pathextstr = strsave(getenv("PATHEXT"));
     if (!pathextstr)
       pathextstr = strsave(PATH_EXT);
     curr_prefix = searchpathext(program_name, pathextstr, getenv("PATH"));
-    a_delete pathextstr;
+    delete[] pathextstr;
   }
+#else /* !_WIN32 */
+  curr_prefix = searchpath(program_name, getenv("PATH"));
   if (!curr_prefix)
     return;
 #endif /* !_WIN32 */
@@ -211,7 +211,9 @@ char *relocatep(const char *path)
     return strsave(path);
   char *relative_path = (char *)path + INSTALLPATHLEN;
   size_t relative_path_len = strlen(relative_path);
-  char *relocated_path = new char[curr_prefix_len + relative_path_len + 1];
+  char *relocated_path = (char *)malloc(curr_prefix_len
+					+ relative_path_len + 1);
+  assert(0 != curr_prefix);
   strcpy(relocated_path, curr_prefix);
   strcat(relocated_path, relative_path);
 #if DEBUG
@@ -234,3 +236,9 @@ char *relocate(const char *path)
 #endif
   return p;
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
