@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989-2014  Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -15,7 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "lib.h"
 
@@ -121,7 +121,7 @@ symbol::symbol(const char *p, int how)
 	   symbol temp(*pp, 1); /* insert it into the new table */
 	   unused(&temp);
 	 }
-    a_delete old_table;
+    delete[] old_table;
     for (pp = table + hc % table_size;
 	 *pp != 0; 
 	 (pp == table ? pp = table + table_size - 1 : --pp))
@@ -150,7 +150,7 @@ symbol concat(symbol s1, symbol s2)
   strcpy(buf, s1.contents());
   strcat(buf, s2.contents());
   symbol res(buf);
-  a_delete buf;
+  delete[] buf;
   return res;
 }
 
