@@ -1,4 +1,4 @@
-/* Copyright (C) 2014  Free Software Foundation, Inc.
+/* Copyright (C) 2014-2020 Free Software Foundation, Inc.
 
 This file is part of groff.
 
@@ -15,9 +15,13 @@ for more details.
 The GNU General Public License version 2 (GPL2) is available in the
 internet at <http://www.gnu.org/licenses/gpl-2.0.txt>. */
 
+#include <config.h>
+
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
 
+char *xmalloc(int n);
+
 char *xmalloc(int n)
 {
     return XtMalloc(n);
