@@ -1,6 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2003, 2007, 2009
-   Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -16,9 +15,14 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <ctype.h>
+#include <stdlib.h>
 #include "eqn.h"
 #include "pbox.h"
 #include "ptable.h"
@@ -52,7 +56,7 @@ struct map entity_table[] = {
   {"*I", "I"},
   {"*k", "&kappa;"},	// ISOgrk3
   {"*K", "K;"},
-  {"*l", "&lamda;"},	// ISOgrk3
+  {"*l", "&lambda;"},	// ISOgrk3
   {"*L", "&Lambda;"},	// ISOgrk3
   {"*m", "&mu;"},	// ISOgrk3
   {"*M", "M"},
@@ -550,10 +554,10 @@ void box::set_spacing_type(char *type)
 {
   int t = lookup_spacing_type(type);
   if (t < 0)
-    error("unrecognised type `%1'", type);
+    error("unrecognised type '%1'", type);
   else
     spacing_type = t;
-  a_delete type;
+  free(type);
 }
 
 char_box::char_box(unsigned char cc)
@@ -648,7 +652,7 @@ special_char_box::special_char_box(const char *t)
 
 special_char_box::~special_char_box()
 {
-  a_delete s;
+  free(s);
 }
 
 void special_char_box::output()
@@ -700,8 +704,8 @@ void set_char_type(const char *type, char *ch)
   int st = lookup_spacing_type(type);
   int ft = lookup_font_type(type);
   if (st < 0 && ft < 0) {
-    error("bad character type `%1'", type);
-    a_delete ch;
+    error("bad character type '%1'", type);
+    delete[] ch;
     return;
   }
   box *b = split_text(ch);
@@ -709,7 +713,7 @@ void set_char_type(const char *type, char *ch)
   delete b;
 }
 
-/* We give primes special treatment so that in ``x' sub 2'', the ``2''
+/* We give primes special treatment so that in "x' sub 2", the "2"
 will be tucked under the prime */
 
 class prime_box : public pointer_box {
@@ -942,7 +946,7 @@ box *split_text(char *text)
 	fb = b;
     }
   }
-  a_delete text;
+  free(text);
   if (lb != 0)
     return lb;
   else if (fb != 0)
