@@ -1,7 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2002, 2003, 2004, 2006,
-                 2007, 2009
-     Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -17,12 +15,12 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "pic.h"
 #include "ptable.h"
 #include "object.h"
-#include "pic_tab.h"
+#include "pic.hpp"
 
 declare_ptable(char)
 implement_ptable(char)
@@ -92,9 +90,14 @@ int file_input::read_line()
     lineno++;
     for (;;) {
       int c = getc(fp);
+      if (c == '\r') {
+	c = getc(fp);
+	if (c != '\n')
+	  lex_error("invalid input character code %1", '\r');
+      }
       if (c == EOF)
 	break;
-      else if (invalid_input_char(c))
+      else if (is_invalid_input_char(c))
 	lex_error("invalid input character code %1", c);
       else {
 	line += char(c);
@@ -145,7 +148,7 @@ macro_input::macro_input(const char *str)
 
 macro_input::~macro_input()
 {
-  a_delete s;
+  free(s);
 }
 
 int macro_input::get()
@@ -205,8 +208,8 @@ argument_macro_input::argument_macro_input(const char *body, int ac, char **av)
 argument_macro_input::~argument_macro_input()
 {
   for (int i = 0; i < argc; i++)
-    a_delete argv[i];
-  a_delete s;
+    free(argv[i]);
+  free(s);
 }
 
 int argument_macro_input::get()
@@ -410,7 +413,7 @@ void interpolate_macro_with_args(const char *body)
 	      argv[argc] = strsave(token_buffer.contents());
 	  }
 	}
-	// for `foo()', argc = 0
+	// for 'foo()', argc = 0
 	if (argc > 0 || c != ')' || i > 0)
 	  if (!ignore)
 	    argc++;
@@ -926,7 +929,7 @@ int get_token(int lookup_flag)
 	  break;
 	}
 	else if (c == EOF) {
-	  error("missing `\"'");
+	  error("missing '\"'");
 	  break;
 	}
 	else if (c == '"') {
@@ -1384,8 +1387,8 @@ for_input::for_input(char *vr, double f, double t,
 
 for_input::~for_input()
 {
-  a_delete var;
-  a_delete body;
+  free(var);
+  free(body);
 }
 
 int for_input::get()
@@ -1401,7 +1404,7 @@ int for_input::get()
     }
     double val;
     if (!lookup_variable(var, &val)) {
-      lex_error("body of `for' terminated enclosing block");
+      lex_error("body of 'for' terminated enclosing block");
       return EOF;
     }
     if (by_is_multiplicative)
@@ -1462,7 +1465,7 @@ void do_copy(const char *filename)
   errno = 0;
   FILE *fp = fopen(filename, "r");
   if (fp == 0) {
-    lex_error("can't open `%1': %2", filename, strerror(errno));
+    lex_error("can't open '%1': %2", filename, strerror(errno));
     return;
   }
   input_stack::push(new file_input(fp, filename));
@@ -1552,8 +1555,8 @@ copy_thru_input::copy_thru_input(const char *b, const char *u)
 
 copy_thru_input::~copy_thru_input()
 {
-  a_delete body;
-  a_delete until;
+  delete[] body;
+  delete[] until;
 }
 
 int copy_thru_input::get()
@@ -1674,7 +1677,7 @@ simple_file_input::~simple_file_input()
 int simple_file_input::get()
 {
   int c = getc(fp);
-  while (invalid_input_char(c)) {
+  while (is_invalid_input_char(c)) {
     error("invalid input character code %1", c);
     c = getc(fp);
   }
@@ -1686,7 +1689,7 @@ int simple_file_input::get()
 int simple_file_input::peek()
 {
   int c = getc(fp);
-  while (invalid_input_char(c)) {
+  while (is_invalid_input_char(c)) {
     error("invalid input character code %1", c);
     c = getc(fp);
   }
@@ -1708,7 +1711,7 @@ void copy_file_thru(const char *filename, const char *body, const char *until)
   errno = 0;
   FILE *fp = fopen(filename, "r");
   if (fp == 0) {
-    lex_error("can't open `%1': %2", filename, strerror(errno));
+    lex_error("can't open '%1': %2", filename, strerror(errno));
     return;
   }
   input *in = new copy_file_thru_input(new simple_file_input(fp, filename),
@@ -2016,7 +2019,7 @@ void yyerror(const char *s)
       if (context[0] == '\n' && context[1] == '\0')
 	error("%1 before newline", s);
       else
-	error("%1 before `%2'", s, context);
+	error("%1 before '%2'", s, context);
     }
     else
       error("%1 at end of picture", s);
@@ -2026,7 +2029,7 @@ void yyerror(const char *s)
       if (context[0] == '\n' && context[1] == '\0')
 	error_with_file_and_line(filename, lineno, "%1 before newline", s);
       else
-	error_with_file_and_line(filename, lineno, "%1 before `%2'",
+	error_with_file_and_line(filename, lineno, "%1 before '%2'",
 				 s, context);
     }
     else
