@@ -1,6 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-1992, 2000, 2001, 2002, 2003, 2006, 2009, 2012
-   Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -16,7 +14,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "pic.h"
 
@@ -26,7 +24,9 @@ extern "C" const char *Version_string;
 output *out;
 char *graphname;		// the picture box name in TeX mode
 
-int flyback_flag;
+bool want_flyback = false;
+// groff pic supports '.PY' to work around mm package stepping on 'PF'.
+bool want_alternate_flyback = false;
 int zero_length_line_flag = 0;
 // Non-zero means we're using a groff driver.
 int driver_extension_flag = 1;
@@ -35,7 +35,7 @@ int safer_flag = 1;
 int command_char = '.';		// the character that introduces lines
 				// that should be passed through transparently
 static int lf_flag = 1;		// non-zero if we should attempt to understand
-				// lines beginning with `.lf'
+				// lines beginning with '.lf'
 
 // Non-zero means a parse error was encountered.
 static int had_parse_error = 0;
@@ -81,7 +81,7 @@ int top_input::get()
     return c;
   }
   int c = getc(fp);
-  while (invalid_input_char(c)) {
+  while (is_invalid_input_char(c)) {
     error("invalid input character code %1", int(c));
     c = getc(fp);
     bol = 0;
@@ -90,13 +90,14 @@ int top_input::get()
     c = getc(fp);
     if (c == 'P') {
       c = getc(fp);
-      if (c == 'F' || c == 'E') {
+      if (c == 'E' || c == 'F' || c == 'Y') {
 	int d = getc(fp);
 	if (d != EOF)
 	  ungetc(d, fp);
 	if (d == EOF || d == ' ' || d == '\n' || compatible_flag) {
 	  eof = 1;
-	  flyback_flag = c == 'F';
+	  want_flyback = (c == 'F');
+	  want_alternate_flyback = (c == 'Y');
 	  return EOF;
 	}
 	push_back[0] = c;
@@ -135,7 +136,7 @@ int top_input::get()
   bol = 0;
   if (c == EOF) {
     eof = 1;
-    error("end of file before .PE or .PF");
+    error("end of file before .PE, .PF, or .PY");
     error_with_file_and_line(current_filename, start_lineno - 1,
 			     ".PS was here");
   }
@@ -153,7 +154,7 @@ int top_input::peek()
   if (push_back[0] != EOF)
     return push_back[0];
   int c = getc(fp);
-  while (invalid_input_char(c)) {
+  while (is_invalid_input_char(c)) {
     error("invalid input character code %1", int(c));
     c = getc(fp);
     bol = 0;
@@ -162,13 +163,14 @@ int top_input::peek()
     c = getc(fp);
     if (c == 'P') {
       c = getc(fp);
-      if (c == 'F' || c == 'E') {
+      if (c == 'E' || c == 'F' || c == 'Y') {
 	int d = getc(fp);
 	if (d != EOF)
 	  ungetc(d, fp);
 	if (d == EOF || d == ' ' || d == '\n' || compatible_flag) {
 	  eof = 1;
-	  flyback_flag = c == 'F';
+	  want_flyback = (c == 'F');
+	  want_alternate_flyback = (c == 'Y');
 	  return EOF;
 	}
 	push_back[0] = c;
@@ -219,9 +221,10 @@ int top_input::get_location(const char **filenamep, int *linenop)
 
 void do_picture(FILE *fp)
 {
-  flyback_flag = 0;
+  want_flyback = false;
   int c;
-  a_delete graphname;
+  if (!graphname)
+    free(graphname);
   graphname = strsave("graph");		// default picture name in TeX mode
   while ((c = getc(fp)) == ' ')
     ;
@@ -241,7 +244,7 @@ void do_picture(FILE *fp)
     if (c == '\n') 
       current_lineno++;
     if (filename.length() == 0)
-      error("missing filename after `<'");
+      error("missing filename after '<'");
     else {
       filename += '\0';
       const char *old_filename = current_filename;
@@ -288,7 +291,7 @@ void do_picture(FILE *fp)
     parse_cleanup();
     lex_cleanup();
 
-    // skip the rest of the .PF/.PE line
+    // skip the rest of the .PE/.PF/.PY line
     while ((c = getc(fp)) != EOF && c != '\n')
       ;
     if (c == '\n')
@@ -307,16 +310,19 @@ void do_file(const char *filename)
     fp = fopen(filename, "r");
     if (fp == 0) {
       delete out;
-      fatal("can't open `%1': %2", filename, strerror(errno));
+      fatal("can't open '%1': %2", filename, strerror(errno));
     }
   }
-  out->set_location(filename, 1);
-  current_filename = filename;
+  string fn(filename);
+  fn += '\0';
+  normalize_for_lf(fn);
+  current_filename = fn.contents();
+  out->set_location(current_filename, 1);
   current_lineno = 1;
   enum { START, MIDDLE, HAD_DOT, HAD_P, HAD_PS, HAD_l, HAD_lf } state = START;
   for (;;) {
     int c = getc(fp);
-    while (invalid_input_char(c)) {
+    while (is_invalid_input_char(c)) {
       error("invalid input character code %1", int(c));
       c = getc(fp);
     }
@@ -464,7 +470,7 @@ void do_whole_file(const char *filename)
     errno = 0;
     fp = fopen(filename, "r");
     if (fp == 0)
-      fatal("can't open `%1': %2", filename, strerror(errno));
+      fatal("can't open '%1': %2", filename, strerror(errno));
   }
   lex_init(new file_input(fp, filename));
   if (yyparse())
@@ -476,13 +482,15 @@ void do_whole_file(const char *filename)
 
 void usage(FILE *stream)
 {
-  fprintf(stream, "usage: %s [ -nvCSU ] [ filename ... ]\n", program_name);
+  fprintf(stream, "usage: %s [-CnSU] [file ...]\n", program_name);
 #ifdef TEX_SUPPORT
-  fprintf(stream, "       %s -t [ -cvzCSU ] [ filename ... ]\n", program_name);
+  fprintf(stream, "usage: %s -t [-cCSUz] [file ...]\n", program_name);
 #endif
 #ifdef FIG_SUPPORT
-  fprintf(stream, "       %s -f [ -v ] [ filename ]\n", program_name);
+  fprintf(stream, "usage: %s -f [-v] [file]\n", program_name);
 #endif
+  fprintf(stream, "usage: %s {-v | --version}\n", program_name);
+  fprintf(stream, "usage: %s --help\n", program_name);
 }
 
 #if defined(__MSDOS__) || defined(__EMX__)
@@ -616,7 +624,13 @@ int main(int argc, char **argv)
     out = make_fig_output();
   else
 #endif
+  {
     out = make_troff_output();
+    printf(".do if !dPS .ds PS\n"
+	   ".do if !dPE .ds PE\n"
+	   ".do if !dPF .ds PF\n"
+	   ".do if !dPY .ds PY\n");
+  }
 #ifdef FIG_SUPPORT
   if (whole_file_flag) {
     if (optind >= argc)
@@ -643,3 +657,8 @@ int main(int argc, char **argv)
   return had_parse_error;
 }
 
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
