@@ -1,7 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2001, 2002, 2003, 2004, 2005,
-                 2006, 2007, 2009
-     Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -17,7 +15,13 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <stdlib.h>
 
 #include "pic.h"
 #include "ptable.h"
@@ -36,7 +40,7 @@ output::output() : args(0), desired_height(0.0), desired_width(0.0)
 
 output::~output()
 {
-  a_delete args;
+  delete[] args;
 }
 
 void output::set_desired_width_height(double wid, double ht)
@@ -47,7 +51,7 @@ void output::set_desired_width_height(double wid, double ht)
 
 void output::set_args(const char *s)
 {
-  a_delete args;
+  delete[] args;
   if (s == 0 || *s == '\0')
     args = 0;
   else
@@ -400,7 +404,7 @@ text_item::text_item(char *t, const char *fn, int ln)
 
 text_item::~text_item()
 {
-  a_delete text;
+  delete[] text;
 }
 
 object_spec::object_spec(object_type t) : type(t)
@@ -439,8 +443,8 @@ object_spec::~object_spec()
     delete tem;
   }
   delete with;
-  a_delete shaded;
-  a_delete outlined;
+  delete[] shaded;
+  delete[] outlined;
 }
 
 class command_object : public object {
@@ -461,7 +465,7 @@ command_object::command_object(char *p, const char *fn, int ln)
 
 command_object::~command_object()
 {
-  a_delete s;
+  delete[] s;
 }
 
 void command_object::print()
@@ -542,7 +546,7 @@ text_piece::text_piece()
 
 text_piece::~text_piece()
 {
-  a_delete text;
+  free(text);
 }
 
 class graphic_object : public object {
@@ -665,7 +669,7 @@ void graphic_object::print_text()
 graphic_object::~graphic_object()
 {
   if (text)
-    ad_delete(ntext) text;
+    delete[] text;
 }
 
 class rectangle_object : public graphic_object {
@@ -1162,7 +1166,7 @@ graphic_object *object_spec::make_move(position *curpos, direction *dirp)
   static position last_move;
   static int have_last_move = 0;
   *dirp = dir;
-  // No need to look at at since `at' attribute sets `from' attribute.
+  // No need to look at at since 'at' attribute sets 'from' attribute.
   position startpos = (flags & HAS_FROM) ? from : *curpos;
   if (!(flags & HAS_SEGMENT)) {
     if ((flags & IS_SAME) && have_last_move)
@@ -1441,7 +1445,7 @@ void spline_object::print()
 
 line_object::~line_object()
 {
-  a_delete v;
+  delete[] v;
 }
 
 linear_object *object_spec::make_line(position *curpos, direction *dirp)
@@ -1449,8 +1453,8 @@ linear_object *object_spec::make_line(position *curpos, direction *dirp)
   static position last_line;
   static int have_last_line = 0;
   *dirp = dir;
-  // We handle `at' only in conjunction with `with', otherwise it is
-  // the same as the `from' attribute.
+  // We handle 'at' only in conjunction with 'with', otherwise it is
+  // the same as the 'from' attribute.
   position startpos;
   if ((flags & HAS_AT) && (flags & HAS_WITH))
     // handled later -- we need the end position
@@ -1504,7 +1508,7 @@ linear_object *object_spec::make_line(position *curpos, direction *dirp)
       s->is_absolute = 1;	// to avoid confusion
     }
   if ((flags & HAS_AT) && (flags & HAS_WITH)) {
-    // `tmpobj' works for arrows and splines too -- we only need positions
+    // 'tmpobj' works for arrows and splines too -- we only need positions
     line_object tmpobj(startpos, endpos, 0, 0);
     position pos = at;
     place offset;
@@ -1960,15 +1964,15 @@ string_list::string_list(char *s)
 
 string_list::~string_list()
 {
-  a_delete str;
+  free(str);
 }
-  
-/* A path is used to hold the argument to the `with' attribute.  For
-   example, `.nw' or `.A.s' or `.A'.  The major operation on a path is to
-   take a place and follow the path through the place to place within the
-   place.  Note that `.A.B.C.sw' will work.
 
-   For compatibility with DWB pic, `with' accepts positions also (this
+/* A path is used to hold the argument to the 'with' attribute.  For
+   example, '.nw' or '.A.s' or '.A'.  The major operation on a path is to
+   take a place and follow the path through the place to place within the
+   place.  Note that '.A.B.C.sw' will work.
+
+   For compatibility with DWB pic, 'with' accepts positions also (this
    is incorrectly documented in CSTR 116). */
 
 path::path(corner c)
@@ -2031,7 +2035,7 @@ int path::follow(const place &pl, place *result) const
   const place *p = &pl;
   for (string_list *lb = label_list; lb; lb = lb->next)
     if (p->obj == 0 || (p = p->obj->find_label(lb->str)) == 0) {
-      lex_error("object does not contain a place `%1'", lb->str);
+      lex_error("object does not contain a place '%1'", lb->str);
       return 0;
     }
   if (crn == 0 || p->obj == 0)
