@@ -1,5 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 2005-2014  Free Software Foundation, Inc.
+/* Copyright (C) 2005-2020 Free Software Foundation, Inc.
      Written by Werner Lemberg (wl@gnu.org)
 
 This file is part of groff.
@@ -15,18 +14,24 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "lib.h"
 
 #include <assert.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <sys/stat.h>
+#ifdef HAVE_UCHARDET
+#include <uchardet/uchardet.h>
+#endif
+
 #include "errarg.h"
 #include "error.h"
 #include "localcharset.h"
 #include "nonposix.h"
 #include "stringclass.h"
+#include "lf.h"
 
 #include <locale.h>
 
@@ -43,10 +48,10 @@ along with this program. If not, see <http://www.gnu.org/licenses/>. */
 
 extern "C" const char *Version_string;
 
-char default_encoding[MAX_VAR_LEN];
+char fallback_encoding[MAX_VAR_LEN];
 char user_encoding[MAX_VAR_LEN];
 char encoding_string[MAX_VAR_LEN];
-int debug_flag = 0;
+bool is_debugging = false;
 int raw_flag = 0;
 
 struct conversion {
@@ -60,22 +65,22 @@ struct conversion {
 //
 // For encodings which don't have a MIME tag we use GNU iconv's encoding
 // names (which also work with the portable GNU libiconv package).  They
-// are marked with `*'.
+// are marked with '*'.
 //
 // Encodings specific to XEmacs and Emacs are marked as such; no mark means
 // that they are used by both Emacs and XEmacs.
 //
-// Encodings marked with `--' are special to Emacs, XEmacs, or other
+// Encodings marked with '--' are special to Emacs, XEmacs, or other
 // applications and shouldn't be used for data exchange.
 //
-// `Not covered' means that the encoding can be handled neither by GNU iconv
+// 'Not covered' means that the encoding can be handled neither by GNU iconv
 // nor by libiconv, or just one of them has support for it.
 //
 // A special case is VIQR encoding: Despite of having a MIME tag it is
 // missing in both libiconv 1.10 and iconv (coming with GNU libc 2.3.6).
 //
-// Finally, we add all aliases of GNU iconv for `ascii', `latin1', and
-// `utf8' to catch those encoding names before iconv is called.
+// Finally, we add all aliases of GNU iconv for 'ascii', 'latin1', and
+// 'utf8' to catch those encoding names before iconv is called.
 //
 // Note that most entries are commented out -- only a small, (rather)
 // reliable and stable subset of encodings is recognized (for coding tags)
@@ -400,9 +405,13 @@ unicode_entity(int u)
   if (u < 0x80)
     putchar(u);
   else {
-    // Handle soft hyphen specially -- it is an input character only,
-    // not a glyph.
-    if (u == 0xAD) {
+    // Handle no-break space and soft hyphen specially--they are input
+    // characters only, not glyphs.  See groff_char(7).
+    if (u == 0xA0) {
+      putchar('\\');
+      putchar('~');
+    }
+    else if (u == 0xAD) {
       putchar('\\');
       putchar('%');
     }
@@ -412,7 +421,7 @@ unicode_entity(int u)
 }
 
 // ---------------------------------------------------------
-// Conversion functions.  All functions take `data', which
+// Conversion functions.  All functions take 'data', which
 // normally holds the first two lines, and a file pointer.
 // ---------------------------------------------------------
 
@@ -444,9 +453,9 @@ struct utf8 {
     FIFTH,
     SIXTH
   } byte;
-  int expected_bytes;
-  int invalid_warning;
-  int incomplete_warning;
+  int expected_byte_count;
+  bool emit_invalid_utf8_warning;
+  bool emit_incomplete_utf8_warning;
   utf8(FILE *);
   ~utf8();
   void add(unsigned char);
@@ -454,8 +463,9 @@ struct utf8 {
   void incomplete();
 };
 
-utf8::utf8(FILE *f) : fp(f), byte(FIRST), expected_bytes(1),
-		      invalid_warning(1), incomplete_warning(1)
+utf8::utf8(FILE *f) : fp(f), byte(FIRST), expected_byte_count(1),
+		      emit_invalid_utf8_warning(true),
+		      emit_incomplete_utf8_warning(true)
 {
   // empty
 }
@@ -476,23 +486,23 @@ utf8::add(unsigned char c)
     else if (c < 0xC0)
       invalid();
     else if (c < 0xE0) {
-      expected_bytes = 2;
+      expected_byte_count = 2;
       byte = SECOND;
     }
     else if (c < 0xF0) {
-      expected_bytes = 3;
+      expected_byte_count = 3;
       byte = SECOND;
     }
     else if (c < 0xF8) {
-      expected_bytes = 4;
+      expected_byte_count = 4;
       byte = SECOND;
     }
     else if (c < 0xFC) {
-      expected_bytes = 5;
+      expected_byte_count = 5;
       byte = SECOND;
     }
     else if (c < 0xFE) {
-      expected_bytes = 6;
+      expected_byte_count = 6;
       byte = SECOND;
     }
     else
@@ -509,7 +519,7 @@ utf8::add(unsigned char c)
     // can't happen
     break;
   case SECOND:
-    if (expected_bytes == 2) {
+    if (expected_byte_count == 2) {
       if (s[0] < 0xC2)
 	invalid();
       else
@@ -521,7 +531,7 @@ utf8::add(unsigned char c)
       byte = THIRD;
     break;
   case THIRD:
-    if (expected_bytes == 3) {
+    if (expected_byte_count == 3) {
       if (!(s[0] >= 0xE1 || s[1] >= 0xA0))
 	invalid();
       else
@@ -535,7 +545,7 @@ utf8::add(unsigned char c)
     break;
   case FOURTH:
     // We reject everything greater than 0x10FFFF.
-    if (expected_bytes == 4) {
+    if (expected_byte_count == 4) {
       if (!((s[0] >= 0xF1 || s[1] >= 0x90)
 	    && (s[0] < 0xF4 || (s[0] == 0xF4 && s[1] < 0x90))))
 	invalid();
@@ -550,7 +560,7 @@ utf8::add(unsigned char c)
       byte = FIFTH;
     break;
   case FIFTH:
-    if (expected_bytes == 5) {
+    if (expected_byte_count == 5) {
       invalid();
       byte = FIRST;
     }
@@ -564,13 +574,16 @@ utf8::add(unsigned char c)
   }
 }
 
+// We use fprintf(stderr) instead of libgroff's debug() because we need
+// to output longs, and libgroff's errprint() doesn't support that.
+
 void
 utf8::invalid()
 {
-  if (debug_flag && invalid_warning) {
-    fprintf(stderr, "  invalid byte(s) found in input stream --\n"
-		    "  each such sequence replaced with 0xFFFD\n");
-    invalid_warning = 0;
+  if (is_debugging && emit_invalid_utf8_warning) {
+    fprintf(stderr, "  invalid UTF-8 sequence(s) in input stream:"
+		    " replacing each such sequence with 0xFFFD\n");
+    emit_invalid_utf8_warning = false;
   }
   unicode_entity(0xFFFD);
   byte = FIRST;
@@ -579,10 +592,10 @@ utf8::invalid()
 void
 utf8::incomplete()
 {
-  if (debug_flag && incomplete_warning) {
-    fprintf(stderr, "  incomplete sequence(s) found in input stream --\n"
-		    "  each such sequence replaced with 0xFFFD\n");
-    incomplete_warning = 0;
+  if (is_debugging && emit_incomplete_utf8_warning) {
+    fprintf(stderr, "  incomplete UTF-8 sequence(s) in input stream:"
+		    " replacing each such sequence with 0xFFFD\n");
+    emit_incomplete_utf8_warning = false;
   }
   unicode_entity(0xFFFD);
   byte = FIRST;
@@ -658,7 +671,7 @@ conversion_iconv(FILE *fp, const string &data, char *enc)
   iconv_t handle = iconv_open(UNICODE, enc);
   if (handle == (iconv_t)-1) {
     if (errno == EINVAL) {
-      error("encoding system `%1' not supported by iconv()", enc);
+      error("encoding system '%1' not supported by iconv()", enc);
       return;
     }
     fatal("iconv_open failed");
@@ -667,7 +680,7 @@ conversion_iconv(FILE *fp, const string &data, char *enc)
   int outbuf[BUFSIZ];
   char *outptr = (char *)outbuf;
   size_t outbytes_left = BUFSIZ * sizeof (int);
-  // Handle `data'.
+  // Handle 'data'.
   char *inptr = (char *)data.contents();
   size_t inbytes_left = data.length();
   char *limit;
@@ -691,13 +704,13 @@ conversion_iconv(FILE *fp, const string &data, char *enc)
 	outbytes_left = BUFSIZ * sizeof (int) - outbytes_left;
       }
       else if (errno == EINVAL) {
-	// `data' ends with partial input sequence.
+	// 'data' ends with partial input sequence.
 	memcpy(inbuf, inptr, inbytes_left);
 	break;
       }
     }
   }
-  // Handle `fp' and switch to `inbuf'.
+  // Handle 'fp' and switch to 'inbuf'.
   size_t read_bytes;
   char *read_start = inbuf + inbytes_left;
   while ((read_bytes = fread(read_start, 1, BUFSIZ - inbytes_left, fp)) > 0) {
@@ -723,7 +736,7 @@ conversion_iconv(FILE *fp, const string &data, char *enc)
 	  outbytes_left = BUFSIZ * sizeof (int) - outbytes_left;
 	}
 	else if (errno == EINVAL) {
-	  // `inbuf' ends with partial input sequence.
+	  // 'inbuf' ends with partial input sequence.
 	  memmove(inbuf, inptr, inbytes_left);
 	  break;
 	}
@@ -748,7 +761,7 @@ conversion_iconv(FILE *fp, const string &data, char *enc)
 // that any normal text file (regardless of the encoding)
 // starts with the bytes which represent a BOM.
 //
-// Return the BOM in string `BOM'; `data' then starts with
+// Return the BOM in string 'BOM'; 'data' then starts with
 // the byte after the BOM.  This function reads (at most)
 // four bytes from the data stream.
 //
@@ -804,9 +817,9 @@ get_BOM(FILE *fp, string &BOM, string &data)
 // ---------------------------------------------------------
 // Get first two lines from input stream.
 //
-// Return string (allocated with `new') without zero bytes
+// Return string (allocated with 'new') without zero bytes
 // or NULL in case no coding tag can occur in the data
-// (which is stored unmodified in `data').
+// (which is stored unmodified in 'data').
 // ---------------------------------------------------------
 char *
 get_tag_lines(FILE *fp, string &data)
@@ -824,14 +837,13 @@ get_tag_lines(FILE *fp, string &data)
   }
   if (newline_count > 1)
     return NULL;
-  int emit_warning = 1;
+  bool emit_warning = true;
   for (int lines = newline_count; lines < 2; lines++) {
     while ((c = getc(fp)) != EOF) {
-      if (c == '\0' && debug_flag && emit_warning) {
-	fprintf(stderr,
-		"  null byte(s) found in input stream --\n"
-		"  search for coding tag might return false result\n");
-	emit_warning = 0;
+      if (c == '\0' && is_debugging && emit_warning) {
+	warning("null byte(s) found in input stream:"
+		" search for coding tag might return false result");
+	emit_warning = false;
       }
       data += char(c);
       if (c == '\n' || c == '\r')
@@ -887,7 +899,7 @@ is_comment_line(char *s)
 //   <variable1>: <value1>; <variable2>: <value2>; ...
 //
 // Leading and trailing blanks are ignored.  There might be
-// more than one blank after `:' and `;'.
+// more than one blank after ':' and ';'.
 //
 // Return position of next value/variable pair or NULL if
 // at end of data.
@@ -905,7 +917,7 @@ get_variable_value_pair(char *d1, char **variable, char **value)
   while (l < MAX_VAR_LEN - 1 && *d1 && !strchr(";: \t", *d1))
     var[l++] = *(d1++);
   var[l] = 0;
-  // Skip everything until `:', `;', or end of data.
+  // Skip everything until ':', ';', or end of data.
   while (*d1 && *d1 != ':' && *d1 != ';')
     d1++;
   val[0] = 0;
@@ -921,7 +933,7 @@ get_variable_value_pair(char *d1, char **variable, char **value)
   while (l < MAX_VAR_LEN - 1 && *d1 && !strchr("; \t", *d1))
     val[l++] = *(d1++);
   val[l] = 0;
-  // Skip everything until `;' or end of data.
+  // Skip everything until ';' or end of data.
   while (*d1 && *d1 != ';')
     d1++;
   if (*d1 == ';')
@@ -936,14 +948,14 @@ get_variable_value_pair(char *d1, char **variable, char **value)
 //
 //   <comment> ... -*-<local variables list>-*-
 //
-// (`...' might be anything).
+// ('...' might be anything).
 //
 // <comment> can be one of the following syntax forms at the
 // beginning of the line:
 //
 //   .\"   .\#   '\"   '\#   \#
 //
-// There can be whitespace after the leading `.' or "'".
+// There can be whitespace after the leading '.' or "'".
 //
 // The local variables list must occur within the first
 // comment block at the very beginning of the data stream.
@@ -960,8 +972,6 @@ get_variable_value_pair(char *d1, char **variable, char **value)
 // Note that null bytes in the data are skipped before applying
 // the algorithm.  This should work even with files encoded as
 // UTF-16 or UTF-32 (or its siblings) in most cases.
-//
-// XXX Add support for tag at the end of buffer.
 // ---------------------------------------------------------
 char *
 check_coding_tag(FILE *fp, string &data)
@@ -986,18 +996,90 @@ check_coding_tag(FILE *fp, string &data)
       d1 = get_variable_value_pair(d1, &variable, &value);
       if (!strcasecmp(variable, "coding")) {
 	*d2 = '-';		// restore '-'
-	a_delete inbuf;
+	free(inbuf);
 	return value;
       }
     }
     *d2 = '-';			// restore '-'
   }
-  a_delete inbuf;
+  free(inbuf);
   return NULL;
 }
 
+char *
+detect_file_encoding(FILE *fp)
+{
+#ifdef HAVE_UCHARDET
+  uchardet_t ud = NULL;
+  struct stat stat_buf;
+  size_t len, read_bytes;
+  char *data = NULL;
+  int res, current_position;
+  const char *charset;
+  char *ret = NULL;
+
+  current_position = ftell(fp);
+  /* Due to BOM and tag detection, we are not at the beginning of the
+     file. */
+  rewind(fp);
+  if (fstat(fileno(fp), &stat_buf) != 0) {
+    error("fstat: %1", strerror(errno));
+    goto end;
+  }
+  len = stat_buf.st_size;
+  if (is_debugging)
+    fprintf(stderr, "  len: %lu\n", (unsigned long)len);
+  if (len == 0)
+    goto end;
+  data = (char *)calloc(len, 1);
+  read_bytes = fread(data, 1, len, fp);
+  if (read_bytes == 0) {
+    error("fread: %1", strerror(errno));
+    goto end;
+  }
+  /* We rewind back to the original position */
+  if (fseek(fp, current_position, SEEK_SET) != 0) {
+    fatal("fseek: %1", strerror(errno));
+    goto end;
+  }
+  ud = uchardet_new();
+  res = uchardet_handle_data(ud, data, len);
+  if (res != 0) {
+    debug("  uchardet_handle_data: error %1\n", res);
+    goto end;
+  }
+  if (is_debugging)
+    fprintf(stderr, "  uchardet read: %lu bytes\n",
+	    (unsigned long)read_bytes);
+  uchardet_data_end(ud);
+  charset = uchardet_get_charset(ud);
+  if (is_debugging) {
+    if (charset)
+       fprintf(stderr, "  charset: %s\n", charset);
+    else
+       fprintf(stderr, "  charset is NULL\n");
+  }
+  /* uchardet 0.0.1 could return an empty string instead of NULL */
+  if (charset && *charset) {
+    ret = (char *)malloc(strlen(charset) + 1);
+    strcpy(ret, charset);
+  }
+
+end:
+  if (ud)
+     uchardet_delete(ud);
+  if (data)
+     free(data);
+
+  return ret;
+#else /* not HAVE_UCHARDET */
+  return NULL;
+#endif /* not HAVE_UCHARDET */
+}
+
 // ---------------------------------------------------------
-// Handle an input file.  If filename is `-' handle stdin.
+// Handle an input file.  If `filename` is "-", read the
+// standard input stream.
 //
 // Return 1 on success, 0 otherwise.
 // ---------------------------------------------------------
@@ -1006,67 +1088,95 @@ do_file(const char *filename)
 {
   FILE *fp;
   string BOM, data;
-  if (strcmp(filename, "-")) {
-    if (debug_flag)
-      fprintf(stderr, "file `%s':\n", filename);
-    fp = fopen(filename, FOPEN_RB);
-    if (!fp) {
-      error("can't open `%1': %2", filename, strerror(errno));
-      return 0;
-    }
+  bool is_seekable = false;
+  string reported_filename;
+
+  // TODO: Consider moving some of this into a `quoted_file_name`
+  // function in libgroff.
+  if (strcmp(filename, "-") == 0) {
+    fp = stdin;
+    reported_filename = string("<standard input>");
   }
   else {
-    if (debug_flag)
-      fprintf(stderr, "standard input:\n");
-    SET_BINARY(fileno(stdin));
-    fp = stdin;
+    fp = fopen(filename, FOPEN_RB);
+    reported_filename = "'" + string(filename) + "'";
+  }
+  if (!fp) {
+    error("can't open %1: %2", reported_filename.contents(),
+	  strerror(errno));
+    return 0;
+  }
+  if (is_debugging)
+    fprintf(stderr, "processing %s\n", reported_filename.contents());
+  if (fseek(fp, 0L, SEEK_SET) == 0)
+    is_seekable = true;
+  else {
+    SET_BINARY(fileno(fp));
+    if (is_debugging)
+      fprintf(stderr, "  stream is not seekable: %s\n",
+	      strerror(errno));
   }
   const char *BOM_encoding = get_BOM(fp, BOM, data);
   // Determine the encoding.
   char *encoding;
+  int must_free_encoding = 0;
   if (user_encoding[0]) {
-    if (debug_flag) {
-      fprintf(stderr, "  user-specified encoding `%s', "
+    if (is_debugging) {
+      fprintf(stderr, "  user-specified encoding '%s', "
 		      "no search for coding tag\n",
 		      user_encoding);
       if (BOM_encoding && strcmp(BOM_encoding, user_encoding))
-	fprintf(stderr, "  but BOM in data stream implies encoding `%s'!\n",
+	fprintf(stderr, "  but BOM in data stream implies encoding '%s'!\n",
 			BOM_encoding);
     }
     encoding = (char *)user_encoding;
   }
   else if (BOM_encoding) {
-    if (debug_flag)
+    if (is_debugging)
       fprintf(stderr, "  found BOM, no search for coding tag\n");
     encoding = (char *)BOM_encoding;
   }
   else {
-    // `check_coding_tag' returns a pointer to a static array (or NULL).
+    // 'check_coding_tag' returns a pointer to a static array (or NULL).
     char *file_encoding = check_coding_tag(fp, data);
     if (!file_encoding) {
-      if (debug_flag)
-	fprintf(stderr, "  no file encoding\n");
-      file_encoding = default_encoding;
+      if (is_debugging)
+	fprintf(stderr, "  no coding tag\n");
+      if (is_seekable)
+         file_encoding = detect_file_encoding(fp);
+      if (!file_encoding) {
+        if (is_debugging)
+          fprintf(stderr, "  could not detect encoding with uchardet\n");
+        file_encoding = fallback_encoding;
+      }
+      else
+        must_free_encoding = 1;
     }
     else
-      if (debug_flag)
-	fprintf(stderr, "  file encoding: `%s'\n", file_encoding);
+      if (is_debugging)
+	fprintf(stderr, "  coding tag: '%s'\n", file_encoding);
     encoding = file_encoding;
   }
   strncpy(encoding_string, encoding, MAX_VAR_LEN - 1);
   encoding_string[MAX_VAR_LEN - 1] = 0;
+  if (must_free_encoding)
+    free(encoding);
   encoding = encoding_string;
   // Translate from MIME & Emacs encoding names to locale encoding names.
   encoding = emacs2mime(encoding_string);
   if (encoding[0] == '\0') {
-    error("encoding `%1' not supported, not a portable encoding",
+    error("encoding '%1' not supported, not a portable encoding",
 	  encoding_string);
     return 0;
   }
-  if (debug_flag)
-    fprintf(stderr, "  encoding used: `%s'\n", encoding);
-  if (!raw_flag)
-    printf(".lf 1 %s\n", filename);
+  if (is_debugging)
+    fprintf(stderr, "  encoding used: '%s'\n", encoding);
+  if (!raw_flag) {
+    string fn(filename);
+    fn += '\0';
+    normalize_for_lf(fn);
+    printf(".lf 1 %s\n", fn.contents());
+  }
   int success = 1;
   // Call converter (converters write to stdout).
   if (!strcasecmp(encoding, "ISO-8859-1"))
@@ -1079,7 +1189,7 @@ do_file(const char *filename)
 #if HAVE_ICONV
     conversion_iconv(fp, BOM + data, encoding);
 #else
-    error("encoding system `%1' not supported", encoding);
+    error("encoding system '%1' not supported", encoding);
     success = 0;
 #endif /* HAVE_ICONV */
   }
@@ -1094,17 +1204,22 @@ do_file(const char *filename)
 void
 usage(FILE *stream)
 {
-  fprintf(stream, "usage: %s [ option ] [ files ]\n"
-		  "\n"
-		  "-d           show debugging messages\n"
-		  "-D encoding  specify default encoding\n"
-		  "-e encoding  specify input encoding\n"
-		  "-h           print this message\n"
-		  "-r           don't add .lf requests\n"
-		  "-v           print version number\n"
-		  "\n"
-		  "The default encoding is `%s'.\n",
-		  program_name, default_encoding);
+  fprintf(stream,
+"usage: %s [-dr] [-D fallback-encoding] [-e encoding] [file ...]\n"
+"usage: %s {-v | --version}\n"
+"usage: %s {-h | --help}\n",
+	  program_name, program_name, program_name);
+  if (stdout == stream) {
+    fprintf(stream,
+"\n"
+"Read each file, convert its encoded characters to a form GNU"
+" troff(1)\n"
+"can interpret, and send the result to the standard output stream.\n"
+"The default fallback encoding is '%s'.  See the preconv(1) manual"
+" page.\n",
+	  fallback_encoding);
+    exit(EXIT_SUCCESS);
+  }
 }
 
 // ---------------------------------------------------------
@@ -1114,16 +1229,16 @@ int
 main(int argc, char **argv)
 {
   program_name = argv[0];
-  // Determine the default encoding.  This must be done before
-  // getopt() is called since the usage message shows the default
+  // Determine the fallback encoding.  This must be done before
+  // getopt() is called since the usage message shows the fallback
   // encoding.
   setlocale(LC_ALL, "");
   char *locale = getlocale(LC_CTYPE);
   if (!locale || !strcmp(locale, "C") || !strcmp(locale, "POSIX"))
-    strcpy(default_encoding, "latin1");
+    strcpy(fallback_encoding, "latin1");
   else {
-    strncpy(default_encoding, locale_charset(), MAX_VAR_LEN - 1);
-    default_encoding[MAX_VAR_LEN - 1] = 0;
+    strncpy(fallback_encoding, locale_charset(), MAX_VAR_LEN - 1);
+    fallback_encoding[MAX_VAR_LEN - 1] = 0;
   }
 
   program_name = argv[0];
@@ -1133,23 +1248,28 @@ main(int argc, char **argv)
     { "version", no_argument, 0, 'v' },
     { NULL, 0, 0, 0 }
   };
-  // Parse the command line options.
+  // Parse the command-line options.
   while ((opt = getopt_long(argc, argv,
 			    "dD:e:hrv", long_options, NULL)) != EOF)
     switch (opt) {
     case 'v':
-      printf("GNU preconv (groff) version %s %s iconv support\n",
+      printf("GNU preconv (groff) version %s %s iconv support and %s uchardet support\n",
 	     Version_string,
 #ifdef HAVE_ICONV
-	     "with"
+	     "with",
 #else
-	     "without"
+	     "without",
 #endif /* HAVE_ICONV */
+#ifdef HAVE_UCHARDET
+             "with"
+#else
+             "without"
+#endif /* HAVE_UCHARDET */
 	    );
       exit(0);
       break;
     case 'd':
-      debug_flag = 1;
+      is_debugging = true;
       break;
     case 'e':
       if (optarg) {
@@ -1161,8 +1281,8 @@ main(int argc, char **argv)
       break;
     case 'D':
       if (optarg) {
-	strncpy(default_encoding, optarg, MAX_VAR_LEN - 1);
-	default_encoding[MAX_VAR_LEN - 1] = 0;
+	strncpy(fallback_encoding, optarg, MAX_VAR_LEN - 1);
+	fallback_encoding[MAX_VAR_LEN - 1] = 0;
       }
       break;
     case 'r':
@@ -1170,7 +1290,6 @@ main(int argc, char **argv)
       break;
     case 'h':
       usage(stdout);
-      exit(0);
       break;
     case '?':
       usage(stderr);
@@ -1180,8 +1299,8 @@ main(int argc, char **argv)
       assert(0);
     }
   int nbad = 0;
-  if (debug_flag)
-    fprintf(stderr, "default encoding: `%s'\n", default_encoding);
+  if (is_debugging)
+    fprintf(stderr, "fallback encoding: '%s'\n", fallback_encoding);
   if (optind >= argc)
     nbad += !do_file("-");
   else
@@ -1192,4 +1311,8 @@ main(int argc, char **argv)
   return nbad != 0;
 }
 
-/* end of preconv.cpp */
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
