@@ -1,7 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2001, 2002, 2003, 2004, 2007,
-                 2008, 2009, 2010
-   Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -17,12 +14,11 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "lib.h"
 
 #include <stdlib.h>
-#include <assert.h>
 #include <ctype.h>
 #include <errno.h>
 
@@ -31,6 +27,7 @@ along with this program. If not, see <http://www.gnu.org/licenses/>. */
 #include "stringclass.h"
 #include "errarg.h"
 #include "error.h"
+#include "lf.h"
 
 // PREFIX and PREFIX_CHAR must be the same.
 #define PREFIX "3"
@@ -100,12 +97,12 @@ class table {
   string *minimum_width;
   int *column_separation;
   char *equal;
-  int left_separation;
-  int right_separation;
+  int left_separation; // from a vertical rule or box border, in ens
+  int right_separation; // from a vertical rule or box border, in ens
   int total_separation;
   int allocated_rows;
   void build_span_list();
-  void compute_expand_width();
+  void compute_overall_width();
   void do_hspan(int r, int c);
   void do_vspan(int r, int c);
   void allocate(int r);
@@ -135,15 +132,19 @@ class table {
 public:
   unsigned flags;
   enum {
-    CENTER       = 0x00000001,
-    EXPAND       = 0x00000002,
-    BOX          = 0x00000004,
-    ALLBOX       = 0x00000008,
-    DOUBLEBOX    = 0x00000010,
-    NOKEEP       = 0x00000020,
-    NOSPACES     = 0x00000040,
-    NOWARN       = 0x00000080,
-    EXPERIMENTAL = 0x80000000	// undocumented; use as a hook for experiments
+    CENTER        = 0x00000001,
+    EXPAND        = 0x00000002,
+    BOX           = 0x00000004,
+    ALLBOX        = 0x00000008,
+    DOUBLEBOX     = 0x00000010,
+    NOKEEP        = 0x00000020,
+    NOSPACES      = 0x00000040,
+    NOWARN        = 0x00000080,
+    // The next few properties help manage nroff mode output.
+    HAS_TOP_VLINE = 0x00000100,
+    HAS_TOP_HLINE = 0x00000200,
+    GAP_EXPAND    = 0x00000400,
+    EXPERIMENTAL  = 0x80000000 // undocumented
     };
   char *expand;
   table(int nc, unsigned flags, int linesize, char decimal_point_char);
@@ -170,3 +171,9 @@ public:
 void set_troff_location(const char *, int);
 
 extern int compatible_flag;
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
