@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989-2014  Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -15,13 +15,13 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 
 #include "troff.h"
 #include "dictionary.h"
-  
-// is `p' a good size for a hash table
+
+// is 'p' a good size for a hash table
 
 static int is_good_size(unsigned int p)
 {
@@ -75,7 +75,7 @@ void *dictionary::lookup(symbol s, void *v)
     for (i = 0; i < old_size; i++)
       if (old_table[i].v != 0)
 	(void)lookup(old_table[i].s, old_table[i].v);
-    a_delete old_table;
+    delete[] old_table;
   }
   return 0;
 }
