@@ -1,5 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 2003-2014  Free Software Foundation, Inc.
+/* Copyright (C) 2003-2020 Free Software Foundation, Inc.
      Written by Gaius Mulley (gaius@glam.ac.uk)
 
 This file is part of groff.
@@ -15,9 +14,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
-#define DEBUGGING
+// mtsm: minimum troff state machine
 
 extern int debug_state;
 
@@ -27,7 +26,9 @@ extern int debug_state;
 #include "mtsm.h"
 #include "env.h"
 
-static int no_of_statems = 0;	// debugging aid
+#if defined(DEBUGGING)
+static int no_of_statems = 0;
+#endif
 
 int_value::int_value()
 : value(0), is_known(0)
@@ -173,8 +174,10 @@ int string_value::differs(string_value compare)
 
 statem::statem()
 {
+#if defined(DEBUGGING)
   issue_no = no_of_statems;
   no_of_statems++;
+#endif
 }
 
 statem::statem(statem *copy)
@@ -188,7 +191,9 @@ statem::statem(statem *copy)
     units_values[i] = copy->units_values[i];
   for (i = 0; i < LAST_STRING; i++)
     string_values[i] = copy->string_values[i];
+#if defined(DEBUGGING)
   issue_no = copy->issue_no;
+#endif
 }
 
 statem::~statem()
@@ -219,10 +224,12 @@ void statem::flush(FILE *fp, statem *compare)
 			     compare->bool_values[MTSM_EOL]);
   bool_values[MTSM_BR].diff(fp, "devtag:.br",
 			    compare->bool_values[MTSM_BR]);
+#if defined(DEBUGGING)
   if (debug_state) {
     fprintf(stderr, "compared state %d\n", compare->issue_no);
     fflush(stderr);
   }
+#endif
 }
 
 void statem::add_tag(int_value_state t, int v)
@@ -366,7 +373,7 @@ mtsm::~mtsm()
 }
 
 /*
- *  push_state - push the current troff state and use `n' as
+ *  push_state - push the current troff state and use 'n' as
  *               the new troff state.
  */
 
@@ -374,8 +381,10 @@ void mtsm::push_state(statem *n)
 {
   if (is_html) {
 #if defined(DEBUGGING)
-    if (debug_state)
-      fprintf(stderr, "--> state %d pushed\n", n->issue_no) ; fflush(stderr);
+    if (debug_state) {
+      fprintf(stderr, "--> state %d pushed\n", n->issue_no);
+      fflush(stderr);
+    }
 #endif
     sp = new stack(n, sp);
   }
@@ -385,8 +394,10 @@ void mtsm::pop_state()
 {
   if (is_html) {
 #if defined(DEBUGGING)
-    if (debug_state)
-      fprintf(stderr, "--> state popped\n") ; fflush(stderr);
+    if (debug_state) {
+      fprintf(stderr, "--> state popped\n");
+      fflush(stderr);
+    }
 #endif
     if (sp == 0)
       fatal("empty state machine stack");
@@ -420,9 +431,11 @@ void mtsm::inherit(statem *s, int reset_bool)
       if (reset_bool)
 	sp->state->bool_values[MTSM_BR].set(0);
       s->bool_values[MTSM_BR].set(1);
+#if defined(DEBUGGING)
       if (debug_state)
 	fprintf(stderr, "inherited br from pushed state %d\n",
 		sp->state->issue_no);
+#endif
     }
     else if (s->bool_values[MTSM_BR].is_known
 	     && s->bool_values[MTSM_BR].value)
@@ -630,3 +643,9 @@ units state_set::val(units_value_state)
 {
   return unitsset;
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
