@@ -1,5 +1,4 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-2014  Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -15,12 +14,11 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program. If not, see <http://www.gnu.org/licenses/>. */
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "lib.h"
 
 #include <ctype.h>
-#include <assert.h>
 #include <stdlib.h>
 #include <errno.h>
 #include "errarg.h"
@@ -33,6 +31,7 @@ extern "C" const char *Version_string;
 
 static void usage(FILE *stream);
 static void usage();
+static void usage(const char *problem);
 static void version();
 static void convert_font(const font_params &, FILE *, FILE *);
 
@@ -42,14 +41,14 @@ static struct {
   const char *name;
   param_t par;
 } param_table[] = {
-  { "x-height", &font_params::x_height },
-  { "fig-height", &font_params::fig_height },
   { "asc-height", &font_params::asc_height },
+  { "body-depth", &font_params::body_depth },
   { "body-height", &font_params::body_height },
   { "cap-height", &font_params::cap_height },
   { "comma-depth", &font_params::comma_depth },
   { "desc-depth", &font_params::desc_depth },
-  { "body-depth", &font_params::body_depth },
+  { "fig-height", &font_params::fig_height },
+  { "x-height", &font_params::x_height },
 };
 
 // These are all in thousandths of an em.
@@ -77,17 +76,20 @@ int main(int argc, char **argv)
     }
   }
   if (argc < 4)
-    usage();
+    usage("insufficient arguments");
+  /* The next couple of usage() calls cannot provide a meaningful
+     diagnostic because we don't know whether sscanf() failed on a
+     required parameter or an option.  A refactor could fix this. */
   int resolution;
   if (sscanf(argv[argc-3], "%d", &resolution) != 1)
     usage();
   if (resolution <= 0)
-    fatal("resolution must be > 0");
+    fatal("resolution must be positive");
   int unitwidth;
   if (sscanf(argv[argc-2], "%d", &unitwidth) != 1)
     usage();
   if (unitwidth <= 0)
-    fatal("unitwidth must be > 0");
+    fatal("unit width must be positive");
   font_params param;
   const char *font = argv[argc-1];
   param.italic = (font[0] != '\0' && strchr(font, '\0')[-1] == 'I');
@@ -106,40 +108,51 @@ int main(int argc, char **argv)
       break;
     }
     if (i + 1 >= argc)
-      usage();
+      usage("option requires argument");
     size_t j;
     for (j = 0;; j++) {
       if (j >= sizeof(param_table)/sizeof(param_table[0]))
-	fatal("parameter `%1' not recognized", argv[i] + 1);
+	fatal("parameter '%1' not recognized", argv[i] + 1);
       if (strcmp(param_table[j].name, argv[i] + 1) == 0)
 	break;
     }
     if (sscanf(argv[i+1], "%d", &(param.*(param_table[j].par))) != 1)
-      fatal("invalid argument `%1'", argv[i+1]);
+      fatal("invalid option argument '%1'", argv[i+1]);
     i++;
-  }    
+  }
   if (argc - i != 3)
-    usage();
+    usage("insufficient arguments");
   errno = 0;
   FILE *infp = fopen(font, "r");
   if (infp == 0)
-    fatal("can't open `%1': %2", font, strerror(errno));
+    fatal("can't open '%1': %2", font, strerror(errno));
   convert_font(param, infp, stdout);
   return 0;
 }
 
 static void usage(FILE *stream)
 {
-  fprintf(stream, "usage: %s [-v] [-param value] ... "
-		  "resolution unitwidth font\n",
-	  program_name);
+  fprintf(stream, "usage: %s", program_name);
+  size_t len = sizeof(param_table)/sizeof(param_table[0]);
+  for (size_t i = 0; i < len; i++)
+    fprintf(stream, " [-%s n]", param_table[i].name);
+  fputs(" resolution unit-width font\n", stream);
+  fprintf(stream, "usage: %s {-v | --version}\n"
+	  "usage: %s --help\n", program_name, program_name);
 }
+
 static void usage()
 {
   usage(stderr);
   exit(1);
 }
 
+static void usage(const char *problem)
+{
+  error("%1", problem);
+  usage();
+}
+
 static void version()
 {
   printf("GNU addftinfo (groff) version %s\n", Version_string);
@@ -157,8 +170,9 @@ static int get_line(FILE *fp, string *p)
   }
   return p->length() > 0;
 }
-  
-static void convert_font(const font_params &param, FILE *infp, FILE *outfp)
+
+static void convert_font(const font_params &param, FILE *infp,
+			 FILE *outfp)
 {
   string s;
   while (get_line(infp, &s)) {
@@ -200,18 +214,24 @@ static void convert_font(const font_params &param, FILE *infp, FILE *outfp)
 		printf(",%d,%d", metric.height, metric.depth);
 	    }
 	    else
-	      printf(",%d,%d,%d", metric.height, metric.depth, metric.ic);
+	      printf(",%d,%d,%d", metric.height, metric.depth,
+		     metric.ic);
 	  }
 	  else
-	    printf(",%d,%d,%d,%d", metric.height, metric.depth, metric.ic,
-		   metric.left_ic);
+	    printf(",%d,%d,%d,%d", metric.height, metric.depth,
+		   metric.ic, metric.left_ic);
 	}
 	else
-	  printf(",%d,%d,%d,%d,%d", metric.height, metric.depth, metric.ic,
-		 metric.left_ic, metric.sk);
+	  printf(",%d,%d,%d,%d,%d", metric.height, metric.depth,
+		 metric.ic, metric.left_ic, metric.sk);
       }
     }
     fputs(p, outfp);
   }
 }
 
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
