@@ -1,28 +1,26 @@
-// -*- C++ -*-
-/* Copyright (C) 1989-1992, 2000, 2001 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2020 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 groff is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
-You should have received a copy of the GNU General Public License along
-with groff; see the file COPYING.  If not, write to the Free Software
-Foundation, 51 Franklin St - Fifth Floor, Boston, MA 02110-1301, USA. */
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "lib.h"
 
-#include <stdlib.h>
-#include <errno.h>
 #include <assert.h>
+#include <errno.h>
+#include <stdlib.h>
 
 #include "errarg.h"
 #include "error.h"
@@ -35,8 +33,11 @@ extern "C" const char *Version_string;
 
 static void usage(FILE *stream)
 {
-  fprintf(stream, "usage: %s [-nv] [-p database] [-i XYZ] [-t N] keys ...\n",
-	  program_name);
+  fprintf(stream,
+          "usage: %s [-n] [-p database] [-i XYZ] [-t N] key ...\n"
+          "usage: %s {-v | --version}\n"
+          "usage: %s --help\n",
+	  program_name, program_name, program_name);
 }
 
 int main(int argc, char **argv)
@@ -56,7 +57,7 @@ int main(int argc, char **argv)
 	 != EOF)
     switch (opt) {
     case 'V':
-      verify_flag = 1;
+      do_verify = true;
       break;
     case 'n':
       search_default = 0;
@@ -69,7 +70,7 @@ int main(int argc, char **argv)
 	char *ptr;
 	long n = strtol(optarg, &ptr, 10);
 	if (n == 0 && ptr == optarg) {
-	  error("bad integer `%1' in `t' option", optarg);
+	  error("bad integer '%1' in 't' option", optarg);
 	  break;
 	}
 	if (n < 1)
@@ -135,3 +136,9 @@ int main(int argc, char **argv)
   }
   return !count;
 }
+
+// Local Variables:
+// fill-column: 72
+// mode: C++
+// End:
+// vim: set cindent noexpandtab shiftwidth=2 textwidth=72:
